import json
import os
from types import SimpleNamespace
from unittest.mock import patch

import pytest
from ckan import model
from ckan.common import _, config
from ckan.tests import factories

from ckanext.feedback.command import feedback
from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.plugin import FeedbackPlugin

engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestPlugin:
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def teardown_method(self, method):
        if os.path.isfile('/srv/app/feedback_config.json'):
            os.remove('/srv/app/feedback_config.json')

    def test_update_config_with_feedback_config_file(self):
        instance = FeedbackPlugin()

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_feedback_config_file is False

        # without .ini file
        feedback_config = {
            'modules': {
                'utilizations': {'enable': True, 'enable_orgs': []},
                'resources': {
                    'enable': True,
                    'enable_orgs': [],
                    'comments': {
                        'repeat_post_limit': {'enable': False, 'enable_orgs': []},
                        'rating': {'enable': False, 'enable_orgs': []},
                    },
                },
                'downloads': {
                    'enable': True,
                    'enable_orgs': [],
                },
                "notice": {
                    "email": {
                        "enable": True,
                        "template_directory": "/opt/ckan/default/src/ckanext-feedback/"
                        "ckanext/feedback/templates/email_notification",
                        "template_utilization": "utilization.text",
                        "template_utilization_comment": "utilization_comment.text",
                        "template_resource_comment": "resource_comment.text",
                        "subject_utilization": "Post a Utilization",
                        "subject_utilization_comment": "Post a Utilization comment",
                        "subject_resource_comment": "Post a Resource comment",
                    }
                },
                "recaptcha": {
                    "enable": True,
                    "publickey": "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
                    "privatekey": "yyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy",
                    "score_threshold": 0.5,
                },
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)

        instance.update_config(config)
        assert instance.is_feedback_config_file is True
        assert config.get('ckan.feedback.utilizations.enable') is True
        assert config.get('ckan.feedback.utilizations.enable_orgs') == []
        assert config.get('ckan.feedback.resources.enable') is True
        assert config.get('ckan.feedback.resources.enable_orgs') == []
        assert (
            config.get('ckan.feedback.resources.comment.repeat_post_limit.enable')
            is False
        )
        assert (
            config.get('ckan.feedback.resources.comment.repeat_post_limit.enable_orgs')
            == []
        )
        assert config.get('ckan.feedback.resources.comment.rating.enable') is False
        assert config.get('ckan.feedback.resources.comment.rating.enable_orgs') == []
        assert config.get('ckan.feedback.downloads.enable') is True
        assert config.get('ckan.feedback.downloads.enable_orgs') == []
        assert config.get('ckan.feedback.notice.email.enable') is True
        assert (
            config.get('ckan.feedback.notice.email.template_directory')
            == "/opt/ckan/default/src/ckanext-feedback/"
            "ckanext/feedback/templates/email_notification"
        )
        assert (
            config.get('ckan.feedback.notice.email.template_utilization')
            == "utilization.text"
        )
        assert (
            config.get('ckan.feedback.notice.email.template_utilization_comment')
            == "utilization_comment.text"
        )
        assert (
            config.get('ckan.feedback.notice.email.template_resource_comment')
            == "resource_comment.text"
        )
        assert (
            config.get('ckan.feedback.notice.email.subject_utilization')
            == "Post a Utilization"
        )
        assert (
            config.get('ckan.feedback.notice.email.subject_utilization_comment')
            == "Post a Utilization comment"
        )
        assert (
            config.get('ckan.feedback.notice.email.subject_resource_comment')
            == "Post a Resource comment"
        )
        assert config.get('ckan.feedback.recaptcha.enable') is True
        assert (
            config.get('ckan.feedback.recaptcha.publickey')
            == "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        )
        assert (
            config.get('ckan.feedback.recaptcha.privatekey')
            == "yyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy"
        )
        assert config.get('ckan.feedback.recaptcha.score_threshold') == 0.5

        # with .ini file enable is opposite from feedback_config.json
        config['ckan.feedback.utilizations.enable'] = False
        config['ckan.feedback.resources.enable'] = False
        config['ckan.feedback.downloads.enable'] = False
        config['ckan.feedback.resources.comment.repeat_post_limit.enable'] = True
        config['ckan.feedback.resources.comment.rating.enable'] = True
        config['ckan.feedback.notice.email.enable'] = True
        config['ckan.feedback.notice.email.template_directory'] = ''
        config['ckan.feedback.notice.email.template_utilization'] = ''
        config['ckan.feedback.notice.email.template_utilization_comment'] = ''
        config['ckan.feedback.notice.email.template_resource_comment'] = ''
        config['ckan.feedback.notice.email.subject_utilization'] = ''
        config['ckan.feedback.notice.email.subject_utilization_comment'] = ''
        config['ckan.feedback.notice.email.subject_resource_comment'] = ''
        config['ckan.feedback.recaptcha.enable'] = True
        config['ckan.feedback.recaptcha.publickey'] = ''
        config['ckan.feedback.recaptcha.privatekey'] = ''
        config['ckan.feedback.recaptcha.score_threshold'] = ''
        instance.update_config(config)
        assert instance.is_feedback_config_file is True
        assert config.get('ckan.feedback.utilizations.enable') is True
        assert config.get('ckan.feedback.resources.enable') is True
        assert (
            config.get('ckan.feedback.resources.comment.repeat_post_limit.enable')
            is False
        )
        assert config.get('ckan.feedback.resources.comment.rating.enable') is False
        assert config.get('ckan.feedback.downloads.enable') is True
        assert config.get('ckan.feedback.notice.email.enable') is True
        assert (
            config.get('ckan.feedback.notice.email.template_directory')
            == "/opt/ckan/default/src/ckanext-feedback/"
            "ckanext/feedback/templates/email_notification"
        )
        assert (
            config.get('ckan.feedback.notice.email.template_utilization')
            == "utilization.text"
        )
        assert (
            config.get('ckan.feedback.notice.email.template_utilization_comment')
            == "utilization_comment.text"
        )
        assert (
            config.get('ckan.feedback.notice.email.template_resource_comment')
            == "resource_comment.text"
        )
        assert (
            config.get('ckan.feedback.notice.email.subject_utilization')
            == "Post a Utilization"
        )
        assert (
            config.get('ckan.feedback.notice.email.subject_utilization_comment')
            == "Post a Utilization comment"
        )
        assert (
            config.get('ckan.feedback.notice.email.subject_resource_comment')
            == "Post a Resource comment"
        )
        assert config.get('ckan.feedback.recaptcha.enable') is True
        assert (
            config.get('ckan.feedback.recaptcha.publickey')
            == "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        )
        assert (
            config.get('ckan.feedback.recaptcha.privatekey')
            == "yyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyyy"
        )
        assert config.get('ckan.feedback.recaptcha.score_threshold') == 0.5

    @patch('ckanext.feedback.plugin.toolkit')
    def test_update_config_attribute_error(self, mock_toolkit):
        instance = FeedbackPlugin()
        feedback_config = {'modules': {}}
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)

        instance.update_config(config)
        mock_toolkit.error_shout.call_count == 4

    @patch('ckanext.feedback.plugin.toolkit')
    def test_update_config_json_decode_error(self, mock_toolkit):
        instance = FeedbackPlugin()
        with open('/srv/app/feedback_config.json', 'w') as f:
            f.write('{"modules":')

        instance.update_config(config)
        mock_toolkit.error_shout.assert_called_once_with(
            'The feedback config file not decoded correctly'
        )

    def test_get_commands(self):
        result = FeedbackPlugin.get_commands(self)
        assert result == [feedback.feedback]

    @patch('ckanext.feedback.plugin.download')
    @patch('ckanext.feedback.plugin.resource')
    @patch('ckanext.feedback.plugin.utilization')
    @patch('ckanext.feedback.plugin.management')
    def test_get_blueprint(
        self,
        mock_management,
        mock_utilization,
        mock_resource,
        mock_download,
    ):
        instance = FeedbackPlugin()

        config['ckan.feedback.utilizations.enable'] = True
        config['ckan.feedback.resources.enable'] = True
        config['ckan.feedback.downloads.enable'] = True
        mock_management.get_management_blueprint.return_value = 'management_bp'
        mock_download.get_download_blueprint.return_value = 'download_bp'
        mock_resource.get_resource_comment_blueprint.return_value = 'resource_bp'
        mock_utilization.get_utilization_blueprint.return_value = 'utilization_bp'

        expected_blueprints = [
            'download_bp',
            'resource_bp',
            'utilization_bp',
            'management_bp',
        ]

        actual_blueprints = instance.get_blueprint()

        assert actual_blueprints == expected_blueprints

        config['ckan.feedback.utilizations.enable'] = False
        config['ckan.feedback.resources.enable'] = False
        config['ckan.feedback.downloads.enable'] = False
        expected_blueprints = ['management_bp']
        actual_blueprints = instance.get_blueprint()

        assert actual_blueprints == expected_blueprints

    @patch('ckanext.feedback.plugin.feedback_config')
    def test_is_enabled_downloads_org(self, mock_feedback_config):
        instance = FeedbackPlugin()
        org_name = 'example_org_name'

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_downloads_org(org_name) is True

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.downloads.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_downloads_org(org_name) is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.downloads.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_downloads_org(org_name) is False

        # with feedback_config_file enable is False and org_name is not in enable_orgs
        feedback_config = {
            'modules': {'downloads': {'enable': False, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_downloads_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is False and org_name is in enable_orgs
        feedback_config = {
            'modules': {'downloads': {'enable': False, 'enable_orgs': [org_name]}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_downloads_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is not in enable_orgs
        feedback_config = {
            'modules': {'downloads': {'enable': True, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_downloads_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is in enable_orgs
        feedback_config = {
            'modules': {'downloads': {'enable': True, 'enable_orgs': [org_name]}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_downloads_org(org_name) is True
        os.remove('/srv/app/feedback_config.json')

    def test_is_enabled_downloads(self):
        instance = FeedbackPlugin()

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_downloads() is True

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.downloads.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_downloads() is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.downloads.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_downloads() is False

        # with feedback_config_file enable is False
        feedback_config = {
            'modules': {'downloads': {'enable': False, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_downloads() is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True
        feedback_config = {
            'modules': {'downloads': {'enable': True, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_downloads() is True

    @patch('ckanext.feedback.plugin.feedback_config')
    def test_is_enabled_resources_org(self, mock_feedback_config):
        instance = FeedbackPlugin()
        org_name = 'example_org_name'

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_resources_org(org_name) is True

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.resources.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_resources_org(org_name) is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.resources.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_resources_org(org_name) is False

        # with feedback_config_file enable is False and org_name is not in enable_orgs
        feedback_config = {
            'modules': {'resources': {'enable': False, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_resources_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is False and org_name is in enable_orgs
        feedback_config = {
            'modules': {'resources': {'enable': False, 'enable_orgs': [org_name]}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_resources_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is not in enable_orgs
        feedback_config = {
            'modules': {'resources': {'enable': True, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_resources_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org
        feedback_config = {
            'modules': {'resources': {'enable': True, 'enable_orgs': [org_name]}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_resources_org(org_name) is True

    def test_is_enabled_resources(self):
        instance = FeedbackPlugin()

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_resources() is True

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.resources.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_resources() is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.resources.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_resources() is False

        # with feedback_config_file enable is False
        feedback_config = {
            'modules': {'resources': {'enable': False, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_resources() is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True
        feedback_config = {
            'modules': {'resources': {'enable': True, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_resources() is True

    @patch('ckanext.feedback.plugin.feedback_config')
    def test_is_enabled_utilizations_org(self, mock_feedback_config):
        instance = FeedbackPlugin()
        org_name = 'example_org_name'

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_utilizations_org(org_name) is True

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.utilizations.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_utilizations_org(org_name) is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.utilizations.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_utilizations_org(org_name) is False

        # with feedback_config_file enable is False and org_name is not in enable_orgs
        feedback_config = {
            'modules': {'utilizations': {'enable': False, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_utilizations_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is False and org_name is in enable_orgs
        feedback_config = {
            'modules': {'utilizations': {'enable': False, 'enable_orgs': [org_name]}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_utilizations_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is not in enable_orgs
        feedback_config = {
            'modules': {'utilizations': {'enable': True, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_utilizations_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is in enable_orgs
        feedback_config = {
            'modules': {'utilizations': {'enable': True, 'enable_orgs': [org_name]}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_utilizations_org(org_name) is True

    def test_is_enabled_utilizations(self):
        instance = FeedbackPlugin()

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_utilizations() is True

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.utilizations.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_utilizations() is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.utilizations.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_utilizations() is False

        # with feedback_config_file enable is False
        feedback_config = {
            'modules': {'utilizations': {'enable': False, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_utilizations() is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True
        feedback_config = {
            'modules': {'utilizations': {'enable': True, 'enable_orgs': []}}
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_utilizations() is True

    @patch('ckanext.feedback.plugin.feedback_config')
    def test_is_disabled_repeat_post_on_resource_org(self, mock_feedback_config):
        instance = FeedbackPlugin()
        org_name = 'example_org_name'

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource_org(org_name) is False

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.resources.comment.repeat_post_limit.enable'] = True
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource_org(org_name) is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.resources.comment.repeat_post_limit.enable'] = False
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource_org(org_name) is False

        # with feedback_config_file enable is False and org_name is not in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {
                        'repeat_post_limit': {'enable': False, 'enable_orgs': []}
                    }
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_disabled_repeat_post_on_resource_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is False and org_name is in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {
                        'repeat_post_limit': {
                            'enable': False,
                            'enable_orgs': [org_name],
                        }
                    }
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_disabled_repeat_post_on_resource_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is not in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {
                        'repeat_post_limit': {'enable': True, 'enable_orgs': []}
                    }
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_disabled_repeat_post_on_resource_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {
                        'repeat_post_limit': {'enable': True, 'enable_orgs': [org_name]}
                    }
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_disabled_repeat_post_on_resource_org(org_name) is True

    def test_is_disabled_repeat_post_on_resource(self):
        instance = FeedbackPlugin()

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource() is False

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.resources.comment.repeat_post_limit.enable'] = True
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource() is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.resources.comment.repeat_post_limit.enable'] = False
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource() is False

        # with feedback_config_file enable is False
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {
                        'repeat_post_limit': {'enable': False, 'enable_orgs': []}
                    }
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource() is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {
                        'repeat_post_limit': {'enable': True, 'enable_orgs': []}
                    }
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_disabled_repeat_post_on_resource() is True

    @patch('ckanext.feedback.plugin.feedback_config')
    def test_is_enabled_rating_org(self, mock_feedback_config):
        instance = FeedbackPlugin()
        org_name = 'example_org_name'

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_rating_org(org_name) is False

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.resources.comment.rating.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_rating_org(org_name) is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.resources.comment.rating.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_rating_org(org_name) is False

        # with feedback_config_file enable is False and org_name is not in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {'rating': {'enable': False, 'enable_orgs': []}}
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_rating_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is False and org_name is in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {'rating': {'enable': False, 'enable_orgs': [org_name]}}
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_rating_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is not in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {'rating': {'enable': True, 'enable_orgs': []}}
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = None
        assert instance.is_enabled_rating_org(org_name) is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True and org_name is in enable_orgs
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {'rating': {'enable': True, 'enable_orgs': [org_name]}}
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        mock_feedback_config.get_organization.return_value = SimpleNamespace(
            **{'name': org_name}
        )
        assert instance.is_enabled_rating_org(org_name) is True

    def test_is_enabled_rating(self):
        instance = FeedbackPlugin()

        # without feedback_config_file and .ini file
        instance.update_config(config)
        assert instance.is_enabled_rating() is False

        # without feedback_config_file, .ini file enable is True
        config['ckan.feedback.resources.comment.rating.enable'] = True
        instance.update_config(config)
        assert instance.is_enabled_rating() is True

        # without feedback_config_file, .ini file enable is False
        config['ckan.feedback.resources.comment.rating.enable'] = False
        instance.update_config(config)
        assert instance.is_enabled_rating() is False

        # with feedback_config_file enable is False
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {'rating': {'enable': False, 'enable_orgs': []}}
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_rating() is False
        os.remove('/srv/app/feedback_config.json')

        # with feedback_config_file enable is True
        feedback_config = {
            'modules': {
                'resources': {
                    'comments': {'rating': {'enable': True, 'enable_orgs': []}}
                }
            }
        }
        with open('/srv/app/feedback_config.json', 'w') as f:
            json.dump(feedback_config, f, indent=2)
        instance.update_config(config)
        assert instance.is_enabled_rating() is True

    def test_is_base_public_folder_bs3(self):
        instance = FeedbackPlugin()
        assert instance.is_base_public_folder_bs3() is False

        config['ckan.base_public_folder'] = 'public-bs3'
        instance.update_config(config)
        assert instance.is_base_public_folder_bs3() is True

    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_downloads_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_utilizations_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_resources_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_rating_org')
    @patch('ckanext.feedback.plugin.download_summary_service')
    @patch('ckanext.feedback.plugin.utilization_summary_service')
    @patch('ckanext.feedback.plugin.resource_summary_service')
    def test_before_dataset_view_with_True(
        self,
        mock_resource_summary_service,
        mock_utilization_summary_service,
        mock_download_summary_service,
        mock_is_enabled_rating_org,
        mock_is_enabled_resources_org,
        mock_is_enabled_utilizations_org,
        mock_is_enabled_downloads_org,
    ):
        instance = FeedbackPlugin()

        mock_is_enabled_rating_org.return_value = False
        mock_is_enabled_resources_org.return_value = True
        mock_is_enabled_utilizations_org.return_value = True
        mock_is_enabled_downloads_org.return_value = True

        mock_resource_summary_service.get_package_comments.return_value = 9999
        mock_resource_summary_service.get_package_rating.return_value = 23.333
        mock_utilization_summary_service.get_package_utilizations.return_value = 9999
        mock_utilization_summary_service.get_package_issue_resolutions.return_value = (
            9999
        )
        mock_download_summary_service.get_package_downloads.return_value = 9999

        dataset = factories.Dataset()

        instance.before_dataset_view(dataset)
        assert dataset['extras'] == [
            {'key': _('Downloads'), 'value': 9999},
            {'key': _('Utilizations'), 'value': 9999},
            {'key': _('Issue Resolutions'), 'value': 9999},
            {'key': _('Comments'), 'value': 9999},
        ]

        mock_is_enabled_rating_org.return_value = True

        dataset['extras'] = []
        instance.before_dataset_view(dataset)
        assert dataset['extras'] == [
            {'key': _('Downloads'), 'value': 9999},
            {'key': _('Utilizations'), 'value': 9999},
            {'key': _('Issue Resolutions'), 'value': 9999},
            {'key': _('Comments'), 'value': 9999},
            {'key': _('Rating'), 'value': 23.3},
        ]

    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_downloads_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_utilizations_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_resources_org')
    def test_before_dataset_view_with_False(
        self,
        mock_is_enabled_resources_org,
        mock_is_enabled_utilizations_org,
        mock_is_enabled_downloads_org,
    ):
        instance = FeedbackPlugin()

        mock_is_enabled_resources_org.return_value = False
        mock_is_enabled_utilizations_org.return_value = False
        mock_is_enabled_downloads_org.return_value = False
        dataset = factories.Dataset()
        dataset['extras'] = [
            'test',
        ]
        before_dataset = dataset

        instance.before_dataset_view(dataset)
        assert before_dataset == dataset

    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_downloads_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_utilizations_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_resources_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_rating_org')
    @patch('ckanext.feedback.plugin.download_summary_service')
    @patch('ckanext.feedback.plugin.utilization_summary_service')
    @patch('ckanext.feedback.plugin.resource_summary_service')
    def test_before_resource_show_with_True(
        self,
        mock_resource_summary_service,
        mock_utilization_summary_service,
        mock_download_summary_service,
        mock_is_enabled_rating_org,
        mock_is_enabled_resources_org,
        mock_is_enabled_utilizations_org,
        mock_is_enabled_downloads_org,
    ):
        instance = FeedbackPlugin()

        mock_is_enabled_rating_org.return_value = False
        mock_is_enabled_resources_org.return_value = True
        mock_is_enabled_utilizations_org.return_value = True
        mock_is_enabled_downloads_org.return_value = True

        mock_resource_summary_service.get_resource_comments.return_value = 9999
        mock_resource_summary_service.get_resource_rating.return_value = 23.333
        mock_utilization_summary_service.get_resource_utilizations.return_value = 9999
        mock_utilization_summary_service.get_resource_issue_resolutions.return_value = (
            9999
        )
        mock_download_summary_service.get_resource_downloads.return_value = 9999

        resource = factories.Resource()

        instance.before_resource_show(resource)
        assert resource[_('Downloads')] == 9999
        assert resource[_('Utilizations')] == 9999
        assert resource[_('Issue Resolutions')] == 9999
        assert resource[_('Comments')] == 9999

        mock_is_enabled_rating_org.return_value = True
        instance.before_resource_show(resource)
        assert resource[_('Rating')] == 23.3

    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_downloads_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_utilizations_org')
    @patch('ckanext.feedback.plugin.FeedbackPlugin.is_enabled_resources_org')
    def test_before_resource_show_with_False(
        self,
        mock_is_enabled_resources_org,
        mock_is_enabled_utilizations_org,
        mock_is_enabled_downloads_org,
    ):
        instance = FeedbackPlugin()

        mock_is_enabled_resources_org.return_value = False
        mock_is_enabled_utilizations_org.return_value = False
        mock_is_enabled_downloads_org.return_value = False
        resource = factories.Resource()
        resource['extras'] = [
            'test',
        ]
        before_resource = resource

        instance.before_resource_show(resource)
        assert before_resource == resource



"""Add columns: rating_comment to resource_comment_summary, and url to utilization.

Revision ID: 2a8c621c22c8
Revises: 40bf9a900ef5
Create Date: 2024-05-22 02:36:08.154409

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = '2a8c621c22c8'
down_revision = '40bf9a900ef5'
branch_labels = None
depends_on = None


def upgrade():
    op.add_column(
        'resource_comment_summary', sa.Column('rating_comment', sa.Integer, default=0)
    )
    op.add_column('utilization', sa.Column('url', sa.Text))


def downgrade():
    op.drop_column('utilization', 'url')
    op.drop_column('resource_comment_summary', 'rating_comment')



import json
from types import SimpleNamespace
from typing import Any

import ckan.model as model
from ckan import plugins
from ckan.common import _, config
from ckan.lib.plugins import DefaultTranslation
from ckan.plugins import toolkit

from ckanext.feedback.command import feedback
from ckanext.feedback.services.common import check
from ckanext.feedback.services.common import config as feedback_config
from ckanext.feedback.services.download import summary as download_summary_service
from ckanext.feedback.services.management import comments as management_comments_service
from ckanext.feedback.services.recaptcha import check as recaptcha_check_service
from ckanext.feedback.services.resource import comment as comment_service
from ckanext.feedback.services.resource import summary as resource_summary_service
from ckanext.feedback.services.utilization import summary as utilization_summary_service
from ckanext.feedback.views import download, management, resource, utilization


class FeedbackPlugin(plugins.SingletonPlugin, DefaultTranslation):
    # Declare class implements
    plugins.implements(plugins.IConfigurer)
    plugins.implements(plugins.IClick)
    plugins.implements(plugins.IBlueprint)
    plugins.implements(plugins.ITemplateHelpers)
    plugins.implements(plugins.ITranslation)
    plugins.implements(plugins.IPackageController, inherit=True)
    plugins.implements(plugins.IResourceController, inherit=True)

    # IConfigurer

    def update_config(self, config):
        # Add this plugin's directories to CKAN's extra paths, so that
        # CKAN will use this plugin's custom files.
        # Paths are relative to this plugin.py file.
        toolkit.add_template_directory(config, 'templates')
        toolkit.add_public_directory(config, 'public')
        toolkit.add_resource('assets', 'feedback')

        # get path to the feedback_config.json file
        # open the file and load the settings
        try:
            feedback_config_path = config.get('ckan.feedback.config_file', '/srv/app')
            with open(f'{feedback_config_path}/feedback_config.json') as json_file:
                feedback_config = json.load(
                    json_file, object_hook=lambda d: SimpleNamespace(**d)
                ).modules
                self.is_feedback_config_file = True

                # the settings related to downloads module
                try:
                    config['ckan.feedback.downloads.enable'] = (
                        feedback_config.downloads.enable
                    )
                    config['ckan.feedback.downloads.enable_orgs'] = (
                        feedback_config.downloads.enable_orgs
                    )
                except AttributeError as e:
                    toolkit.error_shout(e)

                # the settings related to resources module
                try:
                    config['ckan.feedback.resources.enable'] = (
                        feedback_config.resources.enable
                    )
                    config['ckan.feedback.resources.enable_orgs'] = (
                        feedback_config.resources.enable_orgs
                    )
                except AttributeError as e:
                    toolkit.error_shout(e)

                # the settings related to resources comments module
                try:
                    config[
                        'ckan.feedback.resources.comment.repeat_post_limit.enable'
                    ] = feedback_config.resources.comments.repeat_post_limit.enable
                    config[
                        'ckan.feedback.resources.comment.repeat_post_limit.enable_orgs'
                    ] = feedback_config.resources.comments.repeat_post_limit.enable_orgs
                except AttributeError as e:
                    toolkit.error_shout(e)

                # the settings related to resource rating module
                try:
                    config['ckan.feedback.resources.comment.rating.enable'] = (
                        feedback_config.resources.comments.rating.enable
                    )
                    config['ckan.feedback.resources.comment.rating.enable_orgs'] = (
                        feedback_config.resources.comments.rating.enable_orgs
                    )
                except AttributeError as e:
                    toolkit.error_shout(e)

                # the settings related to utilizations module
                try:
                    config['ckan.feedback.utilizations.enable'] = (
                        feedback_config.utilizations.enable
                    )
                    config['ckan.feedback.utilizations.enable_orgs'] = (
                        feedback_config.utilizations.enable_orgs
                    )
                except AttributeError as e:
                    toolkit.error_shout(e)

                # the settings related to recaptcha module
                try:
                    config['ckan.feedback.recaptcha.enable'] = (
                        feedback_config.recaptcha.enable
                    )
                    config['ckan.feedback.recaptcha.privatekey'] = (
                        feedback_config.recaptcha.privatekey
                    )
                    config['ckan.feedback.recaptcha.publickey'] = (
                        feedback_config.recaptcha.publickey
                    )
                    config['ckan.feedback.recaptcha.score_threshold'] = (
                        feedback_config.recaptcha.score_threshold
                    )
                except AttributeError as e:
                    toolkit.error_shout(e)

                # the settings related to email notification function
                try:
                    config['ckan.feedback.notice.email.enable'] = (
                        feedback_config.notice.email.enable
                    )
                    config['ckan.feedback.notice.email.template_directory'] = (
                        feedback_config.notice.email.template_directory
                    )
                    config['ckan.feedback.notice.email.template_utilization'] = (
                        feedback_config.notice.email.template_utilization
                    )
                    config[
                        'ckan.feedback.notice.email.template_utilization_comment'
                    ] = feedback_config.notice.email.template_utilization_comment
                    config['ckan.feedback.notice.email.template_resource_comment'] = (
                        feedback_config.notice.email.template_resource_comment
                    )
                    config['ckan.feedback.notice.email.subject_utilization'] = (
                        feedback_config.notice.email.subject_utilization
                    )
                    config['ckan.feedback.notice.email.subject_utilization_comment'] = (
                        feedback_config.notice.email.subject_utilization_comment
                    )
                    config['ckan.feedback.notice.email.subject_resource_comment'] = (
                        feedback_config.notice.email.subject_resource_comment
                    )
                except AttributeError as e:
                    toolkit.error_shout(e)

        except FileNotFoundError:
            toolkit.error_shout('The feedback config file not found')
            self.is_feedback_config_file = False
        except json.JSONDecodeError:
            toolkit.error_shout('The feedback config file not decoded correctly')

    # IClick

    def get_commands(self):
        return [feedback.feedback]

    # IBlueprint

    # Return a flask Blueprint object to be registered by the extension
    def get_blueprint(self):
        blueprints = []
        if config.get('ckan.feedback.downloads.enable', True):
            blueprints.append(download.get_download_blueprint())
        if config.get('ckan.feedback.resources.enable', True):
            blueprints.append(resource.get_resource_comment_blueprint())
        if config.get('ckan.feedback.utilizations.enable', True):
            blueprints.append(utilization.get_utilization_blueprint())
        blueprints.append(management.get_management_blueprint())
        return blueprints

    # Check production.ini settings
    # Enable/disable the download module
    def is_enabled_downloads_org(self, org_name):
        enable = config.get('ckan.feedback.downloads.enable', True)
        if not self.is_feedback_config_file:
            return toolkit.asbool(enable)
        enable_org = False
        organization = feedback_config.get_organization(org_name)
        if organization is not None:
            enable_org = organization.name in config.get(
                'ckan.feedback.downloads.enable_orgs', []
            )
        downloads_enable = enable and enable_org
        return toolkit.asbool(downloads_enable or not self.is_feedback_config_file)

    def is_enabled_downloads(self):
        enable = config.get('ckan.feedback.downloads.enable', True)
        return toolkit.asbool(enable)

    # Enable/disable the resources module
    def is_enabled_resources_org(self, org_name):
        enable = config.get('ckan.feedback.resources.enable', True)
        if not self.is_feedback_config_file:
            return toolkit.asbool(enable)
        enable_org = False
        organization = feedback_config.get_organization(org_name)
        if organization is not None:
            enable_org = organization.name in config.get(
                'ckan.feedback.resources.enable_orgs', []
            )
        resources_enable = enable and enable_org
        return toolkit.asbool(resources_enable or not self.is_feedback_config_file)

    def is_enabled_resources(self):
        enable = config.get('ckan.feedback.resources.enable', True)
        return toolkit.asbool(enable)

    # Enable/disable the utilizations module
    def is_enabled_utilizations_org(self, org_name):
        enable = config.get('ckan.feedback.utilizations.enable', True)
        if not self.is_feedback_config_file:
            return toolkit.asbool(enable)
        enable_org = False
        organization = feedback_config.get_organization(org_name)
        if organization is not None:
            enable_org = organization.name in config.get(
                'ckan.feedback.utilizations.enable_orgs', []
            )
        utilizations_enable = enable and enable_org
        return toolkit.asbool(utilizations_enable or not self.is_feedback_config_file)

    def is_enabled_utilizations(self):
        enable = config.get('ckan.feedback.utilizations.enable', True)
        return toolkit.asbool(enable)

    # Enable/disable repeat posting on a single resource
    def is_disabled_repeat_post_on_resource_org(self, org_name):
        enable = config.get(
            'ckan.feedback.resources.comment.repeat_post_limit.enable', False
        )
        if not self.is_feedback_config_file:
            return toolkit.asbool(enable)
        enable_org = False
        organization = feedback_config.get_organization(org_name)
        if organization is not None:
            enable_org = organization.name in config.get(
                'ckan.feedback.resources.comment.repeat_post_limit.enable_orgs',
                [],
            )
        repeat_post_limit_enable = enable and enable_org
        return toolkit.asbool(repeat_post_limit_enable)

    def is_disabled_repeat_post_on_resource(self):
        enable = config.get(
            'ckan.feedback.resources.comment.repeat_post_limit.enable', False
        )
        return toolkit.asbool(enable)

    # Enable/disable the rating function
    def is_enabled_rating_org(self, org_name):
        enable = config.get('ckan.feedback.resources.comment.rating.enable', False)
        if not self.is_feedback_config_file:
            return toolkit.asbool(enable)
        enable_org = False
        organization = feedback_config.get_organization(org_name)
        if organization is not None:
            enable_org = organization.name in config.get(
                'ckan.feedback.resources.comment.rating.enable_orgs', []
            )
        rating_enable = enable and enable_org
        return toolkit.asbool(rating_enable)

    def is_enabled_rating(self):
        enable = config.get('ckan.feedback.resources.comment.rating.enable', False)
        return toolkit.asbool(enable)

    def is_base_public_folder_bs3(self):
        base_templates_folder = config.get('ckan.base_public_folder', 'public')
        return base_templates_folder == 'public-bs3'

    # ITemplateHelpers

    def get_helpers(self):
        return {
            'is_enabled_downloads_org': self.is_enabled_downloads_org,
            'is_enabled_downloads': self.is_enabled_downloads,
            'is_enabled_resources_org': self.is_enabled_resources_org,
            'is_enabled_resources': self.is_enabled_resources,
            'is_enabled_utilizations_org': self.is_enabled_utilizations_org,
            'is_enabled_utilizations': self.is_enabled_utilizations,
            'is_disabled_repeat_post_on_resource_org': (
                self.is_disabled_repeat_post_on_resource_org
            ),
            'is_disabled_repeat_post_on_resource': (
                self.is_disabled_repeat_post_on_resource
            ),
            'is_enabled_rating': self.is_enabled_rating,
            'is_enabled_rating_org': self.is_enabled_rating_org,
            'is_organization_admin': check.is_organization_admin,
            'is_base_public_folder_bs3': self.is_base_public_folder_bs3,
            'has_organization_admin_role': check.has_organization_admin_role,
            'get_resource_downloads': download_summary_service.get_resource_downloads,
            'get_package_downloads': download_summary_service.get_package_downloads,
            'get_resource_utilizations': (
                utilization_summary_service.get_resource_utilizations
            ),
            'get_package_utilizations': (
                utilization_summary_service.get_package_utilizations
            ),
            'get_resource_issue_resolutions': (
                utilization_summary_service.get_resource_issue_resolutions
            ),
            'get_package_issue_resolutions': (
                utilization_summary_service.get_package_issue_resolutions
            ),
            'get_comment_reply': comment_service.get_comment_reply,
            'get_resource_comments': resource_summary_service.get_resource_comments,
            'get_package_comments': resource_summary_service.get_package_comments,
            'get_resource_rating': resource_summary_service.get_resource_rating,
            'get_package_rating': resource_summary_service.get_package_rating,
            'get_organization': management_comments_service.get_organization,
            'is_enabled_feedback_recaptcha': (
                recaptcha_check_service.is_enabled_recaptcha
            ),
            'get_feedback_recaptcha_publickey': (
                recaptcha_check_service.get_feedback_recaptcha_publickey
            ),
        }

    # IPackageController

    def before_dataset_view(self, pkg_dict: dict[str, Any]) -> dict[str, Any]:
        package_id = pkg_dict['id']
        owner_org = model.Package.get(package_id).owner_org

        if not pkg_dict['extras']:
            pkg_dict['extras'] = []

        def add_pkg_dict_extras(key: str, value: str):
            pkg_dict['extras'].append({'key': key, 'value': value})

        if self.is_enabled_downloads_org(owner_org):
            add_pkg_dict_extras(
                key=_('Downloads'),
                value=download_summary_service.get_package_downloads(package_id),
            )

        if self.is_enabled_utilizations_org(owner_org):
            add_pkg_dict_extras(
                key=_('Utilizations'),
                value=(
                    utilization_summary_service.get_package_utilizations(package_id)
                ),
            )
            add_pkg_dict_extras(
                key=_('Issue Resolutions'),
                value=(
                    utilization_summary_service.get_package_issue_resolutions(
                        package_id
                    )
                ),
            )

        if self.is_enabled_resources_org(owner_org):
            add_pkg_dict_extras(
                key=_('Comments'),
                value=resource_summary_service.get_package_comments(package_id),
            )
            if self.is_enabled_rating_org(owner_org):
                add_pkg_dict_extras(
                    key=_('Rating'),
                    value=round(
                        resource_summary_service.get_package_rating(package_id), 1
                    ),
                )

        return pkg_dict

    # IResourceController

    def before_resource_show(self, resource_dict: dict[str, Any]) -> dict[str, Any]:
        owner_org = model.Package.get(resource_dict['package_id']).owner_org
        resource_id = resource_dict['id']
        if self.is_enabled_downloads_org(owner_org):
            if _('Downloads') != 'Downloads':
                resource_dict.pop('Downloads', None)
            resource_dict[_('Downloads')] = (
                download_summary_service.get_resource_downloads(resource_id)
            )

        if self.is_enabled_utilizations_org(owner_org):
            if _('Utilizations') != 'Utilizations':
                resource_dict.pop('Utilizations', None)
            resource_dict[_('Utilizations')] = (
                utilization_summary_service.get_resource_utilizations(resource_id)
            )
            if _('Issue Resolutions') != 'Issue Resolutions':
                resource_dict.pop('Issue Resolutions', None)
            resource_dict[_('Issue Resolutions')] = (
                utilization_summary_service.get_resource_issue_resolutions(resource_id)
            )

        if self.is_enabled_resources_org(owner_org):
            if _('Comments') != 'Comments':
                resource_dict.pop('Comments', None)
            resource_dict[_('Comments')] = (
                resource_summary_service.get_resource_comments(resource_id)
            )
            if self.is_enabled_rating_org(owner_org):
                if _('Rating') != 'Rating':
                    resource_dict.pop('Rating', None)
                resource_dict[_('Rating')] = round(
                    resource_summary_service.get_resource_rating(resource_id), 1
                )

        return resource_dict



from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization


# Create new utilization
def create_utilization(resource_id, title, url, description):
    utilization = Utilization(
        resource_id=resource_id,
        title=title,
        url=url,
        description=description,
    )
    session.add(utilization)
    return utilization



import pytest
from ckan import model
from ckan.model.user import User
from ckan.tests import factories

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentSummary,
)
from ckanext.feedback.models.session import session
from ckanext.feedback.services.resource.comment import (
    approve_resource_comment,
    create_resource_comment,
    get_resource_comment_categories,
)
from ckanext.feedback.services.resource.summary import (
    create_resource_summary,
    get_package_comments,
    get_package_rating,
    get_resource_comments,
    get_resource_rating,
    refresh_resource_summary,
)


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestResourceServices:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        engine = model.meta.engine
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_get_package_comments(self):
        resource = factories.Resource()
        assert get_package_comments(resource['package_id']) == 0
        resource_comment_summary = ResourceCommentSummary(
            id=str('test_id'),
            resource_id=resource['id'],
            comment=1,
            rating=1,
            created='2023-03-31 01:23:45.123456',
            updated='2023-03-31 01:23:45.123456',
        )
        session.add(resource_comment_summary)
        session.commit()
        assert get_package_comments(resource['package_id']) == 1

    def test_get_resource_comments(self):
        resource = factories.Resource()
        assert get_resource_comments(resource['id']) == 0
        resource_comment_summary = ResourceCommentSummary(
            id=str('test_id'),
            resource_id=resource['id'],
            comment=1,
            rating=1,
            created='2023-03-31 01:23:45.123456',
            updated='2023-03-31 01:23:45.123456',
        )
        session.add(resource_comment_summary)
        session.commit()
        assert get_resource_comments(resource['id']) == 1

    def test_get_package_rating(self):
        resource = factories.Resource()
        assert get_package_rating(resource['package_id']) == 0
        resource_comment_summary = ResourceCommentSummary(
            id=str('test_id'),
            resource_id=resource['id'],
            comment=1,
            rating=1,
            rating_comment=1,
            created='2023-03-31 01:23:45.123456',
            updated='2023-03-31 01:23:45.123456',
        )
        session.add(resource_comment_summary)
        session.commit()
        assert get_package_rating(resource['package_id']) == 1

    def test_get_resource_rating(self):
        resource = factories.Resource()
        assert get_resource_rating(resource['id']) == 0
        resource_comment_summary = ResourceCommentSummary(
            id=str('test_id'),
            resource_id=resource['id'],
            comment=1,
            rating=1,
            created='2023-03-31 01:23:45.123456',
            updated='2023-03-31 01:23:45.123456',
        )
        session.add(resource_comment_summary)
        session.commit()
        assert get_resource_rating(resource['id']) == 1

    def test_create_resource_summary(self):
        query = session.query(ResourceCommentSummary).all()
        assert len(query) == 0

        resource = factories.Resource()
        create_resource_summary(resource['id'])
        query = session.query(ResourceCommentSummary).all()
        assert len(query) == 1

    def test_refresh_resource_summary(self):
        resource = factories.Resource()
        refresh_resource_summary(resource['id'])
        create_resource_summary(resource['id'])
        session.commit()
        summary = session.query(ResourceCommentSummary).first()
        assert summary.comment == 0
        assert summary.rating == 0
        assert not summary.updated
        category = get_resource_comment_categories().REQUEST
        create_resource_comment(resource['id'], category, 'test', 3)
        session.commit()
        comment_id = session.query(ResourceComment).first().id
        user_id = session.query(User).first().id
        approve_resource_comment(comment_id, user_id)
        refresh_resource_summary(resource['id'])
        session.commit()

        summary = session.query(ResourceCommentSummary).first()
        assert summary.comment == 1
        assert summary.rating == 3.0
        assert summary.updated

        create_resource_comment(resource['id'], category, 'test2', 5)
        session.commit()
        comment_id = session.query(ResourceComment).all()[1].id
        approve_resource_comment(comment_id, user_id)
        refresh_resource_summary(resource['id'])
        session.commit()

        summary = session.query(ResourceCommentSummary).first()
        assert summary.comment == 2
        assert summary.rating == 4.0
        assert summary.updated



import sys

import click
from ckan.model import meta
from ckan.plugins import toolkit

from ckanext.feedback.models.download import DownloadSummary
from ckanext.feedback.models.issue import IssueResolution, IssueResolutionSummary
from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentReply,
    ResourceCommentSummary,
)
from ckanext.feedback.models.utilization import (
    Utilization,
    UtilizationComment,
    UtilizationSummary,
)


@click.group()
def feedback():
    '''CLI tool for ckanext-feedback plugin.'''


@feedback.command(
    name='init', short_help='create tables in ckan db to activate modules.'
)
@click.option(
    '-m',
    '--modules',
    multiple=True,
    type=click.Choice(['utilization', 'resource', 'download']),
    help='specify the module you want to use from utilization, resource, download',
)
def init(modules):
    engine = meta.engine
    try:
        if 'utilization' in modules:
            drop_utilization_tables(engine)
            create_utilization_tables(engine)
            click.secho('Initialize utilization: SUCCESS', fg='green', bold=True)
        elif 'resource' in modules:
            drop_resource_tables(engine)
            create_resource_tables(engine)
            click.secho('Initialize resource: SUCCESS', fg='green', bold=True)
        elif 'download' in modules:
            drop_download_tables(engine)
            create_download_tables(engine)
            click.secho('Initialize download: SUCCESS', fg='green', bold=True)
        else:
            drop_utilization_tables(engine)
            create_utilization_tables(engine)
            drop_resource_tables(engine)
            create_resource_tables(engine)
            drop_download_tables(engine)
            create_download_tables(engine)
            click.secho('Initialize all modules: SUCCESS', fg='green', bold=True)
    except Exception as e:
        toolkit.error_shout(e)
        sys.exit(1)


def drop_utilization_tables(engine):
    IssueResolutionSummary.__table__.drop(engine, checkfirst=True)
    IssueResolution.__table__.drop(engine, checkfirst=True)
    UtilizationSummary.__table__.drop(engine, checkfirst=True)
    UtilizationComment.__table__.drop(engine, checkfirst=True)
    Utilization.__table__.drop(engine, checkfirst=True)


def create_utilization_tables(engine):
    Utilization.__table__.create(engine, checkfirst=True)
    UtilizationComment.__table__.create(engine, checkfirst=True)
    UtilizationSummary.__table__.create(engine, checkfirst=True)
    IssueResolution.__table__.create(engine, checkfirst=True)
    IssueResolutionSummary.__table__.create(engine, checkfirst=True)


def drop_resource_tables(engine):
    ResourceCommentSummary.__table__.drop(engine, checkfirst=True)
    ResourceCommentReply.__table__.drop(engine, checkfirst=True)
    ResourceComment.__table__.drop(engine, checkfirst=True)


def create_resource_tables(engine):
    ResourceComment.__table__.create(engine, checkfirst=True)
    ResourceCommentReply.__table__.create(engine, checkfirst=True)
    ResourceCommentSummary.__table__.create(engine, checkfirst=True)


def drop_download_tables(engine):
    DownloadSummary.__table__.drop(engine, checkfirst=True)


def create_download_tables(engine):
    DownloadSummary.__table__.create(engine, checkfirst=True)



from datetime import datetime

from ckan.model.resource import Resource
from sqlalchemy import func

from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentSummary,
)
from ckanext.feedback.models.session import session


# Get comments of the target package
def get_package_comments(package_id):
    count = (
        session.query(func.sum(ResourceCommentSummary.comment))
        .join(Resource)
        .filter(Resource.package_id == package_id)
        .scalar()
    )
    return count or 0


# Get comments of the target resource
def get_resource_comments(resource_id):
    count = (
        session.query(ResourceCommentSummary.comment)
        .filter(ResourceCommentSummary.resource_id == resource_id)
        .scalar()
    )
    return count or 0


# Get rating of the target package
def get_package_rating(package_id):
    row = (
        session.query(
            func.sum(
                ResourceCommentSummary.rating * ResourceCommentSummary.rating_comment
            ).label('total_rating'),
            func.sum(ResourceCommentSummary.rating_comment).label('rating_comment'),
        )
        .join(Resource)
        .filter(Resource.package_id == package_id)
        .first()
    )
    if row and row.rating_comment and row.rating_comment > 0:
        return row.total_rating / row.rating_comment
    else:
        return 0


# Get rating of the target resource
def get_resource_rating(resource_id):
    rating = (
        session.query(ResourceCommentSummary.rating)
        .filter(ResourceCommentSummary.resource_id == resource_id)
        .scalar()
    )
    return rating or 0


# Create new resource summary
def create_resource_summary(resource_id):
    summary = (
        session.query(ResourceCommentSummary)
        .filter(ResourceCommentSummary.resource_id == resource_id)
        .first()
    )
    if summary is None:
        summary = ResourceCommentSummary(
            resource_id=resource_id,
        )
        session.add(summary)


# Recalculate approved ratings and comments related to the resource summary
def refresh_resource_summary(resource_id):
    total_rating = (
        session.query(
            func.sum(ResourceComment.rating),
        )
        .filter(
            ResourceComment.resource_id == resource_id,
            ResourceComment.approval,
            ResourceComment.rating.isnot(None),
        )
        .scalar()
    )
    if total_rating is None:
        total_rating = 0
    total_comment = (
        session.query(ResourceComment)
        .filter(
            ResourceComment.resource_id == resource_id,
            ResourceComment.approval,
            ResourceComment.rating.isnot(None),
        )
        .count()
    )
    if total_comment > 0:
        rating = total_rating / total_comment
        rating_comment = total_comment
    else:
        rating = 0
        rating_comment = 0

    comment = (
        session.query(ResourceComment)
        .filter(
            ResourceComment.resource_id == resource_id,
            ResourceComment.approval,
            ResourceComment.content.isnot(None),
        )
        .count()
    )

    summary = (
        session.query(ResourceCommentSummary)
        .filter(ResourceCommentSummary.resource_id == resource_id)
        .first()
    )
    if summary is None:
        summary = ResourceCommentSummary(
            resource_id=resource_id,
            rating=rating,
            comment=comment,
            rating_comment=rating_comment,
        )
        session.add(summary)
    else:
        summary.rating = rating
        summary.comment = comment
        summary.rating_comment = rating_comment
        summary.updated = datetime.now()



from ckan.model.group import Group
from ckan.model.package import Package
from ckan.model.resource import Resource
from sqlalchemy import func, or_

from ckanext.feedback.models.issue import IssueResolutionSummary
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization


# Get records from the Utilization table
def get_utilizations(
    id=None,
    keyword=None,
    approval=None,
    admin_owner_orgs=None,
    org_name=None,
    limit=None,
    offset=None,
):
    query = (
        session.query(
            Utilization.id,
            Utilization.title,
            Utilization.comment,
            Utilization.created,
            Utilization.approval,
            Resource.name.label('resource_name'),
            Resource.id.label('resource_id'),
            Package.name.label('package_name'),
            Package.owner_org,
            Group.title.label('organization_name'),
            Group.name,
            func.coalesce(IssueResolutionSummary.issue_resolution, 0).label(
                'issue_resolution'
            ),
        )
        .join(Resource, Utilization.resource)
        .join(Package)
        .join(Group, Package.owner_org == Group.id)
        .outerjoin(IssueResolutionSummary)
        .order_by(Utilization.created.desc())
    )
    if id:
        query = query.filter(or_(Resource.id == id, Package.id == id))
    if keyword:
        query = query.filter(
            or_(
                Utilization.title.like(f'%{keyword}%'),
                Resource.name.like(f'%{keyword}%'),
                Package.name.like(f'%{keyword}%'),
                Package.owner_org.like(f'%{keyword}%'),
                Group.title.like(f'%{keyword}%'),
            )
        )
    if approval is not None:
        query = query.filter(Utilization.approval == approval)
    if admin_owner_orgs is not None:
        query = query.filter(
            or_(Utilization.approval, Package.owner_org.in_(admin_owner_orgs))
        )
    if org_name:
        query = query.filter(Group.name == org_name)

    results = query.limit(limit).offset(offset).all()
    total_count = query.count()

    return results, total_count


def get_organization_name_from_pkg(id):
    package = Package.get(id)
    if package:
        return Group.get(package.owner_org).name
    return None



from unittest.mock import MagicMock, patch

import pytest
from ckan import model
from ckan.common import config

from ckanext.feedback.services.recaptcha import check

engine = model.repo.session.get_bind()


class MockResponse:
    def __init__(self, json_data):
        self._json_data = json_data

    def json(self):
        return self._json_data


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestCheck:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()

    @patch('ckanext.feedback.services.recaptcha.check.requests')
    def test_recaptcha(self, mock_requests):
        config['ckan.feedback.recaptcha.enable'] = True
        config['ckan.feedback.recaptcha.privatekey'] = 'test_private_key'

        mock_request = MagicMock()
        mock_request.form.get.return_value = 'test_recaptcha_response'

        mock_requests.get.return_value = MockResponse({"success": True, "score": 0.9})
        assert check.is_recaptcha_verified(mock_request) is True

    def test_recaptcha_is_disable(self):
        config['ckan.feedback.recaptcha.enable'] = False
        mock_request = MagicMock()
        assert check.is_recaptcha_verified(mock_request) is True

    @patch('ckanext.feedback.services.recaptcha.check.logger')
    @patch('ckanext.feedback.services.recaptcha.check.requests')
    def test_recaptcha_without_no_recaptcha_response(self, mock_requests, mock_logger):
        config['ckan.feedback.recaptcha.enable'] = True

        mock_request = MagicMock()
        mock_request.form.get.return_value = None
        assert check.is_recaptcha_verified(mock_request) is False
        mock_logger.warning.assert_called_once_with("not recaptcha_response")

    @patch('ckanext.feedback.services.recaptcha.check.logger')
    @patch('ckanext.feedback.services.recaptcha.check.requests')
    def test_recaptcha_without_no_privatekey(self, mock_requests, mock_logger):
        config['ckan.feedback.recaptcha.enable'] = True
        config['ckan.feedback.recaptcha.privatekey'] = ''

        mock_request = MagicMock()
        mock_request.form.get.return_value = 'test_recaptcha_response'
        mock_requests.get.return_value = MockResponse({"success": True, "score": 0.9})
        assert check.is_recaptcha_verified(mock_request) is False
        mock_logger.warning.assert_called_once_with("not recaptcha_private_key")

    @patch('ckanext.feedback.services.recaptcha.check.logger')
    @patch('ckanext.feedback.services.recaptcha.check.requests')
    def test_recaptcha_without_not_success(self, mock_requests, mock_logger):
        config['ckan.feedback.recaptcha.enable'] = True
        config['ckan.feedback.recaptcha.privatekey'] = 'test_private_key'

        mock_request = MagicMock()
        mock_request.form.get.return_value = 'test_recaptcha_response'
        data = {"success": False}

        mock_requests.get.return_value = MockResponse(data)
        assert check.is_recaptcha_verified(mock_request) is False
        mock_logger.warning.assert_called_once_with(f"not success:{data}")

    @patch('ckanext.feedback.services.recaptcha.check.logger')
    @patch('ckanext.feedback.services.recaptcha.check.requests')
    def test_recaptcha_without_score(self, mock_requests, mock_logger):
        config['ckan.feedback.recaptcha.enable'] = True
        config['ckan.feedback.recaptcha.privatekey'] = 'test_private_key'

        mock_request = MagicMock()
        mock_request.form.get.return_value = 'test_recaptcha_response'
        data = {"success": True, "score": 0.1}

        mock_requests.get.return_value = MockResponse(data)
        assert check.is_recaptcha_verified(mock_request) is False
        mock_logger.warning.assert_called_once_with(
            f"Score is below the threshold:{data}:score_threshold={0.5}"
        )

    @patch('ckanext.feedback.services.recaptcha.check.logger')
    @patch('ckanext.feedback.services.recaptcha.check.requests')
    def test_recaptcha_without_keyerror(self, mock_requests, mock_logger):
        config['ckan.feedback.recaptcha.enable'] = True
        config['ckan.feedback.recaptcha.privatekey'] = 'test_private_key'

        mock_request = MagicMock()
        mock_request.form.get.return_value = 'test_recaptcha_response'
        mock_requests.get.return_value = MockResponse({"score": None})
        assert check.is_recaptcha_verified(mock_request) is False
        mock_logger.error.assert_called_once_with("KeyError")

    def test_get_feedback_recaptcha_publickey(self):
        config['ckan.feedback.recaptcha.publickey'] = 'test_publick_key'
        ret_value = check.get_feedback_recaptcha_publickey()
        assert ret_value == 'test_publick_key'



import logging

import ckan.views.resource as resource
from ckan.common import config
from flask import Blueprint

from ckanext.feedback.controllers.download import DownloadController
from ckanext.feedback.services.common import config as feedback_config
from ckanext.feedback.services.resource.comment import get_resource
from ckanext.feedback.views.error_handler import add_error_handler

log = logging.getLogger(__name__)

blueprint = Blueprint(
    'download',
    __name__,
    url_prefix='/dataset/<id>/resource',
    url_defaults={'package_type': 'dataset'},
)

# Add target page URLs to rules and add each URL to the blueprint
blueprint.add_url_rule(
    '/<resource_id>/download/<filename>', view_func=DownloadController.extended_download
)
blueprint.add_url_rule(
    '/<resource_id>/download',
    'download',
    view_func=DownloadController.extended_download,
)


@add_error_handler
def get_download_blueprint():
    return blueprint


# Handler to Use When Called from External Extensions
def download(package_type, id, resource_id, filename=None):
    if filename is None:
        filename = get_resource(resource_id).Resource.url

    if config.get('ckan.feedback.downloads.enable', True):
        handler = DownloadController.extended_download
    else:
        handler = feedback_config.download_handler()
        if not handler:
            log.debug('Use default CKAN callback for resource.download')
            handler = resource.download
    return handler(
        package_type=package_type,
        id=id,
        resource_id=resource_id,
        filename=filename,
    )



import pytest
from ckan import model

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.services.utilization import validate

engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestUtilizationDetailsService:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_validate_with_valid_url(self):
        example_valid_url = 'https://example.com'
        result = validate.validate_url(example_valid_url)
        assert result is None

    def test_validate_with_invalid_url(self):
        example_invalid_url = 'invalid_url'
        result = validate.validate_url(example_invalid_url)
        assert result == 'Please provide a valid URL'

    def test_validate_with_invalid_url_len(self):
        example_domain = 'ex'
        for _ in range(10):
            example_domain += example_domain
        result = validate.validate_url('https://' + example_domain + '.com')
        assert result == 'Please keep the URL length below 2048'

    def test_validate_with_valid_title(self):
        example_valid_title = 'example title'
        result = validate.validate_title(example_valid_title)
        assert result is None

    def test_validate_with_invalid_title(self):
        example_invalid_title = (
            'example title'
            'example title'
            'example title'
            'example title'
            'example title'
        )
        result = validate.validate_title(example_invalid_title)
        assert result == 'Please keep the title length below 50'

    def test_validate_with_valid_description(self):
        example_valid_description = 'example description'
        result = validate.validate_description(example_valid_description)
        assert result is None

    def test_validate_with_invalid_description(self):
        example_invalid_description = 'ex'
        for _ in range(11):
            example_invalid_description += example_invalid_description
        result = validate.validate_description(example_invalid_description)
        assert result == 'Please keep the description length below 2000'

    def test_validate_with_valid_comment(self):
        example_valid_comment = 'example comment'
        result = validate.validate_comment(example_valid_comment)
        assert result is None

    def test_validate_with_invalid_comment(self):
        example_invalid_comment = 'ex'
        for _ in range(9):
            example_invalid_comment += example_invalid_comment
        result = validate.validate_comment(example_invalid_comment)
        assert result == 'Please keep the comment length below 1000'



from unittest.mock import patch

import pytest
from ckan import model
from flask import Blueprint, Flask
from psycopg2.errors import UndefinedTable
from sqlalchemy.exc import ProgrammingError

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.views.error_handler import add_error_handler

engine = model.repo.session.get_bind()


def dummy_func_with_undefined_table(**kwargs):
    raise ProgrammingError(statement='', params='', orig=UndefinedTable('', '', ''))


def dummy_func_with_programming_error(**kwargs):
    raise ProgrammingError(statement='', params='', orig=None)


def dummy_func_with_exception(**kwargs):
    raise Exception()


@add_error_handler
def function():
    blueprint = Blueprint(
        'test',
        __name__,
        url_prefix='/',
    )
    blueprint.add_url_rule(
        '/test_undefined_table',
        view_func=dummy_func_with_undefined_table,
    )
    blueprint.add_url_rule(
        '/test_programming_error',
        view_func=dummy_func_with_programming_error,
    )
    blueprint.add_url_rule('/test_exception', view_func=dummy_func_with_exception)
    return blueprint


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestErrorHandler:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def setup_method(self, method):
        self.app = Flask(__name__)
        self.app.config['TESTING'] = True
        self.app.register_blueprint(function())

    @patch('ckanext.feedback.views.error_handler.session', autospec=True)
    @patch('ckanext.feedback.views.error_handler.log', autospec=True)
    def test_handle_programming_error_with_undefined_table(
        self, mock_log, mock_session
    ):
        with self.app.test_client() as client:
            with pytest.raises(ProgrammingError):
                client.get('/test_undefined_table')
            mock_log.error.assert_called_once()
            mock_session.rollback.assert_called_once()

    @patch('ckanext.feedback.views.error_handler.session', autospec=True)
    def test_handle_programming_error(self, mock_session):
        with self.app.test_client() as client:
            with pytest.raises(ProgrammingError):
                client.get('/test_programming_error')
            mock_session.rollback.assert_called_once()

    @patch('ckanext.feedback.views.error_handler.session', autospec=True)
    def test_handle_exception(self, mock_session):
        with self.app.test_client() as client:
            with pytest.raises(Exception):
                client.get('/test_exception')
            mock_session.rollback.assert_called_once()



from unittest.mock import patch

import pytest
from ckan import model
from click.testing import CliRunner

from ckanext.feedback.command.feedback import feedback
from ckanext.feedback.models.download import DownloadSummary
from ckanext.feedback.models.issue import IssueResolution, IssueResolutionSummary
from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentReply,
    ResourceCommentSummary,
)
from ckanext.feedback.models.utilization import (
    Utilization,
    UtilizationComment,
    UtilizationSummary,
)

engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestFeedbackCommand:
    @classmethod
    def setup_class(cls):
        model.repo.metadata.clear()
        model.repo.init_db()

    def teardown_class(cls):
        model.repo.metadata.reflect()

    def setup_method(self, method):
        self.runner = CliRunner()

    def teardown_method(self, method):
        model.repo.metadata.drop_all(
            engine,
            [
                Utilization.__table__,
                UtilizationComment.__table__,
                UtilizationSummary.__table__,
                IssueResolution.__table__,
                IssueResolutionSummary.__table__,
                ResourceComment.__table__,
                ResourceCommentReply.__table__,
                ResourceCommentSummary.__table__,
                DownloadSummary.__table__,
            ],
            checkfirst=True,
        )

    def test_feedback_default(self):
        result = self.runner.invoke(feedback, ['init'])
        assert 'Initialize all modules: SUCCESS' in result.output
        assert engine.has_table(Utilization.__table__)
        assert engine.has_table(UtilizationComment.__table__)
        assert engine.has_table(UtilizationSummary.__table__)
        assert engine.has_table(IssueResolution.__table__)
        assert engine.has_table(IssueResolutionSummary.__table__)
        assert engine.has_table(ResourceComment.__table__)
        assert engine.has_table(ResourceCommentReply.__table__)
        assert engine.has_table(ResourceCommentSummary.__table__)
        assert engine.has_table(DownloadSummary.__table__)

    def test_feedback_utilization(self):
        result = self.runner.invoke(
            feedback,
            ['init', '--modules', 'utilization'],
        )
        assert 'Initialize utilization: SUCCESS' in result.output
        assert engine.has_table(Utilization.__table__)
        assert engine.has_table(UtilizationComment.__table__)
        assert engine.has_table(UtilizationSummary.__table__)
        assert engine.has_table(IssueResolution.__table__)
        assert engine.has_table(IssueResolutionSummary.__table__)
        assert not engine.has_table(ResourceComment.__table__)
        assert not engine.has_table(ResourceCommentReply.__table__)
        assert not engine.has_table(ResourceCommentSummary.__table__)
        assert not engine.has_table(DownloadSummary.__table__)

    def test_feedback_resource(self):
        result = self.runner.invoke(feedback, ['init', '--modules', 'resource'])
        assert 'Initialize resource: SUCCESS' in result.output
        assert not engine.has_table(Utilization.__table__)
        assert not engine.has_table(UtilizationComment.__table__)
        assert not engine.has_table(UtilizationSummary.__table__)
        assert not engine.has_table(IssueResolution.__table__)
        assert not engine.has_table(IssueResolutionSummary.__table__)
        assert engine.has_table(ResourceComment.__table__)
        assert engine.has_table(ResourceCommentReply.__table__)
        assert engine.has_table(ResourceCommentSummary.__table__)
        assert not engine.has_table(DownloadSummary.__table__)

    def test_feedback_download(self):
        result = self.runner.invoke(feedback, ['init', '--modules', 'download'])
        assert 'Initialize download: SUCCESS' in result.output
        assert not engine.has_table(Utilization.__table__)
        assert not engine.has_table(UtilizationComment.__table__)
        assert not engine.has_table(UtilizationSummary.__table__)
        assert not engine.has_table(IssueResolution.__table__)
        assert not engine.has_table(IssueResolutionSummary.__table__)
        assert not engine.has_table(ResourceComment.__table__)
        assert not engine.has_table(ResourceCommentReply.__table__)
        assert not engine.has_table(ResourceCommentSummary.__table__)
        assert engine.has_table(DownloadSummary.__table__)

    def test_feedback_session_error(self):
        with patch(
            'ckanext.feedback.command.feedback.create_utilization_tables',
            side_effect=Exception('Error message'),
        ):
            result = self.runner.invoke(feedback, ['init'])

        assert result.exit_code != 0
        assert not engine.has_table(Utilization.__table__)
        assert not engine.has_table(UtilizationComment.__table__)
        assert not engine.has_table(UtilizationSummary.__table__)
        assert not engine.has_table(IssueResolution.__table__)
        assert not engine.has_table(IssueResolutionSummary.__table__)
        assert not engine.has_table(ResourceComment.__table__)
        assert not engine.has_table(ResourceCommentReply.__table__)
        assert not engine.has_table(ResourceCommentSummary.__table__)
        assert not engine.has_table(DownloadSummary.__table__)



from flask import Blueprint

from ckanext.feedback.controllers import management
from ckanext.feedback.views.error_handler import add_error_handler

blueprint = Blueprint('management', __name__, url_prefix='/management')

# Add target page URLs to rules and add each URL to the blueprint
rules = [
    (
        '/comments',
        'comments',
        management.ManagementController.comments,
        {'methods': ['GET']},
    ),
    (
        '/approve_bulk_utilization_comments',
        'approve_bulk_utilization_comments',
        management.ManagementController.approve_bulk_utilization_comments,
        {'methods': ['POST']},
    ),
    (
        '/approve_bulk_resource_comments',
        'approve_bulk_resource_comments',
        management.ManagementController.approve_bulk_resource_comments,
        {'methods': ['POST']},
    ),
    (
        '/delete_bulk_utilization_comments',
        'delete_bulk_utilization_comments',
        management.ManagementController.delete_bulk_utilization_comments,
        {'methods': ['POST']},
    ),
    (
        '/delete_bulk_resource_comments',
        'delete_bulk_resource_comments',
        management.ManagementController.delete_bulk_resource_comments,
        {'methods': ['POST']},
    ),
]
for rule, endpoint, view_func, *others in rules:
    options = next(iter(others), {})
    blueprint.add_url_rule(rule, endpoint, view_func, **options)


@add_error_handler
def get_management_blueprint():
    return blueprint



import logging
from datetime import datetime

from ckan.model import Resource
from sqlalchemy import func

from ckanext.feedback.models.issue import IssueResolutionSummary
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization, UtilizationSummary

log = logging.getLogger(__name__)


# Get utilization summary count of the target package
def get_package_utilizations(package_id):
    count = (
        session.query(func.sum(UtilizationSummary.utilization))
        .join(Resource)
        .filter(Resource.package_id == package_id)
        .scalar()
    )
    return count or 0


# Get utilization summary count of the target resource
def get_resource_utilizations(resource_id):
    count = (
        session.query(UtilizationSummary.utilization)
        .filter(UtilizationSummary.resource_id == resource_id)
        .scalar()
    )
    return count or 0


# Create new utilizaton summary
def create_utilization_summary(resource_id):
    summary = (
        session.query(UtilizationSummary)
        .filter(UtilizationSummary.resource_id == resource_id)
        .first()
    )
    if summary is None:
        summary = UtilizationSummary(
            resource_id=resource_id,
        )
        session.add(summary)


# Recalculate approved utilization related to the utilization summary
def refresh_utilization_summary(resource_id):
    count = (
        session.query(Utilization)
        .filter(
            Utilization.resource_id == resource_id,
            Utilization.approval,
        )
        .count()
    )
    summary = (
        session.query(UtilizationSummary)
        .filter(UtilizationSummary.resource_id == resource_id)
        .first()
    )
    if summary is None:
        summary = UtilizationSummary(
            resource_id=resource_id,
            utilization=count,
        )
        session.add(summary)
    else:
        summary.utilization = count
        summary.updated = datetime.now()


def get_package_issue_resolutions(package_id):
    count = (
        session.query(func.sum(IssueResolutionSummary.issue_resolution))
        .join(Utilization)
        .join(Resource)
        .filter(Resource.package_id == package_id)
        .scalar()
    )
    return count or 0


def get_resource_issue_resolutions(resource_id):
    count = (
        session.query(func.sum(IssueResolutionSummary.issue_resolution))
        .join(Utilization)
        .filter(Utilization.resource_id == resource_id)
        .scalar()
    )
    return count or 0


def increment_issue_resolution_summary(utilization_id):
    issue_resolution_summary = (
        session.query(IssueResolutionSummary)
        .filter(IssueResolutionSummary.utilization_id == utilization_id)
        .first()
    )
    if issue_resolution_summary is None:
        issue_resolution_summary = IssueResolutionSummary(
            utilization_id=utilization_id,
            issue_resolution=1,
        )
        session.add(issue_resolution_summary)
    else:
        issue_resolution_summary.issue_resolution = (
            issue_resolution_summary.issue_resolution + 1
        )
        issue_resolution_summary.updated = datetime.now()



import logging

from ckan.model.group import Group
from ckan.plugins import toolkit
from werkzeug.utils import import_string

from ckanext.feedback.models.session import session

log = logging.getLogger(__name__)

CONFIG_HANDLER_PATH = 'ckan.feedback.download_handler'


def get_organization(org_id=None):
    return session.query(Group.name.label('name')).filter(Group.id == org_id).first()


def download_handler():
    handler_path = toolkit.config.get(CONFIG_HANDLER_PATH)
    if handler_path:
        handler = import_string(handler_path, silent=True)
    else:
        handler = None
        log.warning(f'Missing {CONFIG_HANDLER_PATH} config option.')

    return handler



import uuid
from datetime import datetime

import ckan.tests.factories as factories
import pytest
from ckan import model

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.issue import IssueResolutionSummary
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization, UtilizationSummary
from ckanext.feedback.services.utilization.summary import (
    create_utilization_summary,
    get_package_issue_resolutions,
    get_package_utilizations,
    get_resource_issue_resolutions,
    get_resource_utilizations,
    increment_issue_resolution_summary,
    refresh_utilization_summary,
)


def get_utilization_summary(resource_id):
    return (
        session.query(UtilizationSummary)
        .filter(UtilizationSummary.resource_id == resource_id)
        .all()
    )


def get_issue_resolution_summary(utilization_id):
    return (
        session.query(IssueResolutionSummary)
        .filter(IssueResolutionSummary.utilization_id == utilization_id)
        .first()
    )


def register_utilization(id, resource_id, title, description, approval):
    utilization = Utilization(
        id=id,
        resource_id=resource_id,
        title=title,
        description=description,
        approval=approval,
    )
    session.add(utilization)


def resister_issue_resolution_summary(id, utilization_id, created, updated):
    issue_resolution_summary = IssueResolutionSummary(
        id=id,
        utilization_id=utilization_id,
        issue_resolution=1,
        created=created,
        updated=updated,
    )
    session.add(issue_resolution_summary)


engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestUtilizationDetailsService:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_get_package_utilizations(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'
        register_utilization(id, resource['id'], title, description, False)

        get_package_utilizations(dataset['id']) == 1

    def test_get_resource_utilizations(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'
        register_utilization(id, resource['id'], title, description, False)

        get_resource_utilizations(resource['id']) == 1

    @pytest.mark.freeze_time(datetime(2000, 1, 2, 3, 4))
    def test_create_utilization_summary(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'
        register_utilization(id, resource['id'], title, description, False)

        assert len(get_utilization_summary(resource['id'])) == 0

        create_utilization_summary(resource['id'])

        assert len(get_utilization_summary(resource['id'])) == 1

        create_utilization_summary(resource['id'])

        assert len(get_utilization_summary(resource['id'])) == 1

    def test_refresh_utilization_summary(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        title = 'test title'
        description = 'test description'

        register_utilization(
            str(uuid.uuid4()), resource['id'], title, description, False
        )

        assert len(get_utilization_summary(resource['id'])) == 0

        refresh_utilization_summary(resource['id'])

        assert len(get_utilization_summary(resource['id'])) == 1
        assert get_utilization_summary(resource['id'])[0].utilization == 0

        register_utilization(
            str(uuid.uuid4()), resource['id'], title, description, True
        )

        refresh_utilization_summary(resource['id'])

        assert len(get_utilization_summary(resource['id'])) == 1
        assert get_utilization_summary(resource['id'])[0].utilization == 1

    def test_get_package_issue_resolutions(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'
        time = datetime.now()

        register_utilization(utilization_id, resource['id'], title, description, True)

        assert get_package_issue_resolutions(dataset['id']) == 0

        resister_issue_resolution_summary(str(uuid.uuid4()), utilization_id, time, time)

        assert get_package_issue_resolutions(dataset['id']) == 1

    def test_get_resource_issue_resolutions(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        utilization_id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'
        time = datetime.now()

        register_utilization(utilization_id, resource['id'], title, description, True)

        assert get_resource_issue_resolutions(resource['id']) == 0

        resister_issue_resolution_summary(str(uuid.uuid4()), utilization_id, time, time)

        assert get_resource_issue_resolutions(resource['id']) == 1

    def test_increment_issue_resolution_summary(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        id = str(uuid.uuid4())
        title = 'test title'
        description = 'test description'

        register_utilization(id, resource['id'], title, description, True)

        increment_issue_resolution_summary(id)

        assert get_issue_resolution_summary(id).issue_resolution == 1

        increment_issue_resolution_summary(id)

        assert get_issue_resolution_summary(id).issue_resolution == 2



from unittest.mock import patch

import pytest
from ckan import model
from ckan.common import config
from ckan.tests import factories
from flask import Flask

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.download import DownloadSummary
from ckanext.feedback.models.session import session
from ckanext.feedback.views.download import download


def get_downloads(resource_id):
    count = (
        session.query(DownloadSummary.download)
        .filter(DownloadSummary.resource_id == resource_id)
        .scalar()
    )
    return count


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestDownloadController:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        engine = model.meta.engine
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def setup_method(self, method):
        self.app = Flask(__name__)

    @patch('ckanext.feedback.views.download.DownloadController.extended_download')
    def test_download_true(self, mock_extended_download):
        resource = factories.Resource()
        mock_extended_download.return_value = mock_extended_download

        # with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'document'}):
        download(
            'package_type', resource['package_id'], resource['id'], resource['url']
        )
        mock_extended_download.assert_called_once_with(
            package_type='package_type',
            id=resource['package_id'],
            resource_id=resource['id'],
            filename=resource['url'],
        )

    @patch('ckanext.feedback.views.download.feedback_config.download_handler')
    @patch('ckanext.feedback.views.download.resource.download')
    def test_download_false_with_not_set_download_handler(
        self, mock_download, mock_download_handler
    ):
        resource = factories.Resource()
        mock_download_handler.return_value = None
        config['ckan.feedback.downloads.enable'] = False

        download(
            'package_type', resource['package_id'], resource['id'], resource['url']
        )
        mock_download.assert_called_once_with(
            package_type='package_type',
            id=resource['package_id'],
            resource_id=resource['id'],
            filename=resource['url'],
        )

    @patch('ckanext.feedback.views.download.feedback_config.download_handler')
    @patch('ckanext.feedback.views.download.DownloadController.extended_download')
    def test_download_false_with_set_download_handler(
        self, mock_extended_download, mock_download_handler
    ):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        mock_download_handler.return_value = mock_extended_download
        config['ckan.feedback.downloads.enable'] = False

        download('package_type', resource['package_id'], resource['id'], None)
        mock_download_handler.assert_called_once_with()



import uuid

import ckan.tests.factories as factories
import pytest
from ckan import model

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization
from ckanext.feedback.services.utilization.edit import (
    delete_utilization,
    get_resource_details,
    get_utilization_details,
    update_utilization,
)


def get_registered_utilization(id):
    return session.query(Utilization).filter(Utilization.id == id).first()


def register_utilization(id, resource_id, title, url, description, approval):
    utilization = Utilization(
        id=id,
        resource_id=resource_id,
        title=title,
        url=url,
        description=description,
        approval=approval,
    )
    session.add(utilization)


engine = model.repo.session.get_bind()


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestUtilizationDetailsService:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_get_utilization_details(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(id, resource['id'], title, url, description, False)

        result = get_utilization_details(id)
        utilization = get_registered_utilization(id)
        assert result == utilization

    def test_get_resource_details(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        result = get_resource_details(resource['id'])

        assert result.resource_name == resource['name']
        assert result.resource_id == resource['id']
        assert result.package_name == dataset['name']

    def test_update_utilization(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(id, resource['id'], title, url, description, False)

        updated_title = 'test updated title'
        updated_url = 'test updated url'
        updated_description = 'test updated description'

        update_utilization(id, updated_title, updated_url, updated_description)

        utilization = get_registered_utilization(id)

        assert utilization.title == updated_title
        assert utilization.url == updated_url
        assert utilization.description == updated_description

    def test_delete_utilization(self):
        dataset = factories.Dataset()
        resource = factories.Resource(package_id=dataset['id'])

        id = str(uuid.uuid4())
        title = 'test title'
        url = 'test url'
        description = 'test description'
        register_utilization(id, resource['id'], title, url, description, False)

        assert get_registered_utilization(id).id == id

        delete_utilization(id)

        assert get_registered_utilization(id) is None



from ckan.model.package import Package
from ckan.model.resource import Resource

from ckanext.feedback.models.session import session
from ckanext.feedback.models.utilization import Utilization


# Get details from the Utilization record
def get_utilization_details(utilization_id):
    return session.query(Utilization).get(utilization_id)


# Get details from the Resource record
def get_resource_details(resource_id):
    return (
        session.query(
            Resource.name.label('resource_name'),
            Resource.id.label('resource_id'),
            Package.name.label('package_name'),
        )
        .join(Package)
        .filter(Resource.id == resource_id)
        .first()
    )


# Update utilization
def update_utilization(utilization_id, title, url, description):
    utilization = session.query(Utilization).get(utilization_id)
    utilization.title = title
    utilization.url = url
    utilization.description = description


# Delete utilization
def delete_utilization(utilization_id):
    utilization = session.query(Utilization).get(utilization_id)
    session.delete(utilization)



import logging
from functools import partial
from typing import Any, Iterable, Optional
from urllib.parse import urlencode

from ckan.common import config, request
from ckan.lib.helpers import helper_functions as h
from typing_extensions import TypeAlias

log = logging.getLogger(__name__)

Params: TypeAlias = "list[tuple[str, Any]]"


def _encode_params(params: Iterable[tuple[str, Any]]):
    return [(k, v.encode('utf-8') if isinstance(v, str) else str(v)) for k, v in params]


def url_with_params(url: str, params: Params) -> str:
    params = _encode_params(params)
    return url + '?' + urlencode(params)


def search_url(params: Params, endpoint) -> str:
    url = h.url_for(endpoint)
    return url_with_params(url, params)


def _pager_url(params_nopage: Params, endpoint, page: Optional[int] = None) -> str:
    params = list(params_nopage)
    params.append(('page', page))
    return search_url(params, endpoint)


def get_pagination_value(endpoint):
    page = h.get_page_number(request.args)

    limit = config.get('ckan.datasets_per_page')
    offset = (page - 1) * limit

    params_nopage = []
    for k, v in request.args.items(multi=True):
        if k != 'page':
            params_nopage += [(k, v)]

    pager_url = partial(_pager_url, params_nopage, endpoint=endpoint)

    return page, limit, offset, pager_url



import ckan.logic.validators as validators


def validate_url(url):
    errors = {'key': []}
    context = {}
    validators.url_validator('key', {'key': url}, errors, context)
    if errors['key']:
        return 'Please provide a valid URL'

    if 2048 < len(url):
        return 'Please keep the URL length below 2048'
    return


def validate_title(title):
    if 50 < len(title):
        return 'Please keep the title length below 50'
    return


def validate_description(description):
    if 2000 < len(description):
        return 'Please keep the description length below 2000'
    return


def validate_comment(comment):
    if 1000 < len(comment):
        return 'Please keep the comment length below 1000'
    return



import logging

import requests
from ckan.common import config
from ckan.plugins import toolkit
from ckan.types import Request

logger = logging.getLogger(__name__)


class CaptchaError(ValueError):
    pass


def is_enabled_recaptcha():
    enable = config.get('ckan.feedback.recaptcha.enable', False)
    return toolkit.asbool(enable)


def get_feedback_recaptcha_publickey() -> str:
    return config.get('ckan.feedback.recaptcha.publickey')


def _check_recaptcha_v3_base(request: Request) -> None:
    '''Check a user's recaptcha submission is valid, and raise CaptchaError
    on failure using discreet data'''
    client_ip_address = request.remote_addr or 'Unknown IP Address'
    recaptcha_response = request.form.get('g-recaptcha-response', '')
    if not recaptcha_response:
        logger.warning('not recaptcha_response')
        raise CaptchaError()

    recaptcha_private_key = config.get('ckan.feedback.recaptcha.privatekey')
    if not recaptcha_private_key:
        logger.warning('not recaptcha_private_key')
        raise CaptchaError()

    # reCAPTCHA v3
    recaptcha_server_name = 'https://www.google.com/recaptcha/api/siteverify'

    # recaptcha_response_field will be unicode if there are foreign chars in
    # the user input. So we need to encode it as utf8 before urlencoding or
    # we get an exception.
    params = {
        'secret': recaptcha_private_key,
        'remoteip': client_ip_address,
        'response': recaptcha_response.encode('utf8'),
    }

    timeout = config.get('ckan.requests.timeout')

    data = requests.get(recaptcha_server_name, params, timeout=timeout).json()
    score_threshold = float(config.get('ckan.feedback.recaptcha.score_threshold', 0.5))

    try:
        if not data['success']:
            logger.warning(f'not success:{data}')
            raise CaptchaError()
        if data['score'] < score_threshold:
            logger.warning(
                f'Score is below the threshold:{data}:score_threshold={score_threshold}'
            )
            raise CaptchaError()
    except KeyError:
        # Something weird with recaptcha response
        logger.error("KeyError")
        raise CaptchaError()

    logger.info(f'reCAPTCHA verification passed successfully:{data}')


def is_recaptcha_verified(request: Request):
    if is_enabled_recaptcha():
        try:
            _check_recaptcha_v3_base(request)
        except CaptchaError:
            return False
    return True



import pytest
from ckan import model
from ckan.model.user import User
from ckan.tests import factories

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.resource_comment import (
    ResourceComment,
    ResourceCommentCategory,
)
from ckanext.feedback.models.session import session
from ckanext.feedback.services.resource.comment import (
    approve_resource_comment,
    create_reply,
    create_resource_comment,
    get_comment_reply,
    get_cookie,
    get_resource,
    get_resource_comment_categories,
    get_resource_comments,
)


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestComments:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        engine = model.meta.engine
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def test_get_resource(self):
        organization_dict = factories.Organization(
            name='org_name',
        )
        package = factories.Dataset(owner_org=organization_dict['id'])
        resource = factories.Resource(package_id=package['id'])
        session.commit()
        assert get_resource(resource['id'])
        assert get_resource(resource['id']).organization_id == package['id']
        assert get_resource(resource['id']).organization_name == "org_name"

    def test_get_resource_comments(self):
        assert not get_resource_comments()
        organization = factories.Organization()
        package = factories.Dataset(owner_org=organization['id'])
        resource = factories.Resource(package_id=package['id'])
        limit = 20
        offset = 0

        category = get_resource_comment_categories().REQUEST
        create_resource_comment(resource['id'], category, 'test', 1)
        session.commit()
        assert get_resource_comments()
        assert get_resource_comments(resource['id'])
        assert get_resource_comments(resource['id'], None, [organization['id']])
        assert not get_resource_comments('test')
        assert not get_resource_comments(resource['id'], True)
        assert get_resource_comments(
            limit=limit,
            offset=offset,
        )

    def test_create_resource_comment(self):
        pass

    def test_get_resource_comment_categories(self):
        assert get_resource_comment_categories() == ResourceCommentCategory

    def test_approve_resource_comment(self):
        resource = factories.Resource()
        category = get_resource_comment_categories().REQUEST
        create_resource_comment(resource['id'], category, 'test', 1)
        session.commit()
        comment_id = session.query(ResourceComment).first().id
        user_id = session.query(User).first().id

        assert not get_resource_comments(resource['id'])[0].approval

        approve_resource_comment(comment_id, user_id)
        session.commit()
        assert get_resource_comments(resource['id'])[0].approval

    def test_get_comment_reply(self):
        pass

    def test_create_reply(self):
        resource = factories.Resource()
        category = get_resource_comment_categories().REQUEST
        create_resource_comment(resource['id'], category, 'test', 1)
        comment_id = session.query(ResourceComment).first().id
        user_id = session.query(User).first().id
        assert not get_comment_reply(comment_id)
        create_reply(comment_id, 'test_reply', user_id)
        session.commit()
        assert get_comment_reply(comment_id)

    def test_get_cookie(self):
        resource = factories.Resource()
        assert not get_cookie(resource['id'])



import uuid
from datetime import datetime

from ckan.model.user import User
from sqlalchemy import TIMESTAMP, Column, ForeignKey, Integer, Text
from sqlalchemy.orm import relationship

from ckanext.feedback.models.session import Base


class IssueResolution(Base):
    __tablename__ = 'issue_resolution'
    id = Column(Text, default=uuid.uuid4, primary_key=True, nullable=False)
    utilization_id = Column(
        Text,
        ForeignKey('utilization.id', onupdate='CASCADE', ondelete='CASCADE'),
        nullable=False,
    )
    description = Column(Text)
    created = Column(TIMESTAMP, default=datetime.now)
    creator_user_id = Column(
        Text, ForeignKey('user.id', onupdate='CASCADE', ondelete='SET NULL')
    )

    utilization = relationship('Utilization', back_populates='issue_resolutions')
    creator_user = relationship(User)


class IssueResolutionSummary(Base):
    __tablename__ = 'issue_resolution_summary'
    id = Column(Text, default=uuid.uuid4, primary_key=True, nullable=False)
    utilization_id = Column(
        Text,
        ForeignKey('utilization.id', onupdate='CASCADE', ondelete='CASCADE'),
        nullable=False,
    )
    issue_resolution = Column(Integer)
    created = Column(TIMESTAMP, default=datetime.now)
    updated = Column(TIMESTAMP)

    utilization = relationship('Utilization', back_populates='issue_resolution_summary')



import enum
import uuid
from datetime import datetime

from ckan.common import _
from ckan.model.resource import Resource
from ckan.model.user import User
from sqlalchemy import BOOLEAN, TIMESTAMP, Column, Enum, ForeignKey, Integer, Text
from sqlalchemy.orm import relationship

from ckanext.feedback.models.session import Base


class UtilizationCommentCategory(enum.Enum):
    REQUEST = _('Request')
    QUESTION = _('Question')
    THANK = _('Thank')


class Utilization(Base):
    __tablename__ = 'utilization'
    id = Column(Text, default=uuid.uuid4, primary_key=True, nullable=False)
    resource_id = Column(
        Text,
        ForeignKey('resource.id', onupdate='CASCADE', ondelete='CASCADE'),
        nullable=False,
    )
    title = Column(Text)
    url = Column(Text)
    description = Column(Text)
    comment = Column(Integer, default=0)
    created = Column(TIMESTAMP, default=datetime.now)
    approval = Column(BOOLEAN, default=False)
    approved = Column(TIMESTAMP)
    approval_user_id = Column(
        Text, ForeignKey('user.id', onupdate='CASCADE', ondelete='SET NULL')
    )

    resource = relationship(Resource)
    approval_user = relationship(User)
    comments = relationship(
        'UtilizationComment', back_populates='utilization', cascade='all, delete-orphan'
    )
    issue_resolutions = relationship(
        'IssueResolution', back_populates='utilization', cascade='all, delete-orphan'
    )
    issue_resolution_summary = relationship(
        'IssueResolutionSummary',
        back_populates='utilization',
        cascade='all, delete-orphan',
    )


class UtilizationComment(Base):
    __tablename__ = 'utilization_comment'
    id = Column(Text, default=uuid.uuid4, primary_key=True, nullable=False)
    utilization_id = Column(
        Text,
        ForeignKey('utilization.id', onupdate='CASCADE', ondelete='CASCADE'),
        nullable=False,
    )
    category = Column(Enum(UtilizationCommentCategory), nullable=False)
    content = Column(Text)
    created = Column(TIMESTAMP, default=datetime.now)
    approval = Column(BOOLEAN, default=False)
    approved = Column(TIMESTAMP)
    approval_user_id = Column(
        Text, ForeignKey('user.id', onupdate='CASCADE', ondelete='SET NULL')
    )

    utilization = relationship('Utilization', back_populates='comments')
    approval_user = relationship(User)


class UtilizationSummary(Base):
    __tablename__ = 'utilization_summary'
    id = Column(Text, default=uuid.uuid4, primary_key=True, nullable=False)
    resource_id = Column(
        Text,
        ForeignKey('resource.id', onupdate='CASCADE', ondelete='CASCADE'),
        nullable=False,
    )
    utilization = Column(Integer, default=0)
    created = Column(TIMESTAMP, default=datetime.now)
    updated = Column(TIMESTAMP)

    resource = relationship(Resource)



import pytest
from ckan import model
from ckan.tests import factories

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.models.download import DownloadSummary
from ckanext.feedback.models.session import session
from ckanext.feedback.services.download.summary import (
    get_package_downloads,
    get_resource_downloads,
    increment_resource_downloads,
)


def get_downloads(resource_id):
    count = (
        session.query(DownloadSummary.download)
        .filter(DownloadSummary.resource_id == resource_id)
        .scalar()
    )
    return count


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestDownloadServices:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        engine = model.meta.engine
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)
        session.commit()

    def test_increment_resource_downloads(self):
        resource = factories.Resource()
        increment_resource_downloads(resource['id'])
        assert get_downloads(resource['id']) == 1
        increment_resource_downloads(resource['id'])
        assert get_downloads(resource['id']) == 2

    def test_get_package_download(self):
        resource = factories.Resource()
        assert get_package_downloads(resource['package_id']) == 0
        download_summary = DownloadSummary(
            id=str('test_id'),
            resource_id=resource['id'],
            download=1,
            created='2023-03-31 01:23:45.123456',
            updated='2023-03-31 01:23:45.123456',
        )
        session.add(download_summary)
        session.commit()
        assert get_package_downloads(resource['package_id']) == 1

    def test_get_resource_download(self):
        resource = factories.Resource()
        assert get_resource_downloads(resource['id']) == 0
        download_summary = DownloadSummary(
            id=str('test_id'),
            resource_id=resource['id'],
            download=1,
            created='2023-03-31 01:23:45.123456',
            updated='2023-03-31 01:23:45.123456',
        )
        session.add(download_summary)
        session.commit()
        assert get_resource_downloads(resource['id']) == 1



import logging

import ckan.model as model
from ckan.common import _, config, current_user, g, request
from ckan.lib import helpers
from ckan.logic import get_action
from ckan.plugins import toolkit
from flask import make_response, redirect, url_for

import ckanext.feedback.services.resource.comment as comment_service
import ckanext.feedback.services.resource.summary as summary_service
import ckanext.feedback.services.resource.validate as validate_service
from ckanext.feedback.controllers.pagination import get_pagination_value
from ckanext.feedback.models.session import session
from ckanext.feedback.services.common.check import (
    check_administrator,
    has_organization_admin_role,
)
from ckanext.feedback.services.common.send_mail import send_email
from ckanext.feedback.services.recaptcha.check import is_recaptcha_verified

log = logging.getLogger(__name__)


class ResourceController:
    # Render HTML pages
    # resource_comment/<resource_id>
    @staticmethod
    def comment(resource_id, category='', content=''):
        approval = True
        resource = comment_service.get_resource(resource_id)
        if not isinstance(current_user, model.User):
            # if the user is not logged in, display only approved comments
            approval = True
        elif (
            has_organization_admin_role(resource.Resource.package.owner_org)
            or current_user.sysadmin
        ):
            # if the user is an organization admin or a sysadmin, display all comments
            approval = None

        page, limit, offset, _ = get_pagination_value('resource_comment.comment')

        comments, total_count = comment_service.get_resource_comments(
            resource_id, approval, limit=limit, offset=offset
        )
        categories = comment_service.get_resource_comment_categories()
        cookie = comment_service.get_cookie(resource_id)
        context = {'model': model, 'session': session, 'for_view': True}
        package = get_action('package_show')(
            context, {'id': resource.Resource.package_id}
        )
        g.pkg_dict = {'organization': {'name': resource.organization_name}}

        return toolkit.render(
            'resource/comment.html',
            {
                'resource': resource.Resource,
                'pkg_dict': package,
                'categories': categories,
                'cookie': cookie,
                'selected_category': category,
                'content': content,
                'page': helpers.Page(
                    collection=comments,
                    page=page,
                    item_count=total_count,
                    items_per_page=limit,
                ),
            },
        )

    # resource_comment/<resource_id>/comment/new
    @staticmethod
    def create_comment(resource_id):
        package_name = request.form.get('package_name', '')
        category = None
        content = None
        if request.form.get('comment-content'):
            content = request.form.get('comment-content')
            category = request.form.get('category')
        rating = None
        if request.form.get('rating'):
            rating = int(request.form.get('rating'))
        if not (category and content):
            toolkit.abort(400)

        if not is_recaptcha_verified(request):
            helpers.flash_error(_(u'Bad Captcha. Please try again.'), allow_html=True)
            return ResourceController.comment(resource_id, category, content)

        if message := validate_service.validate_comment(content):
            helpers.flash_error(
                _(message),
                allow_html=True,
            )
            return ResourceController.comment(resource_id, category, content)

        comment_service.create_resource_comment(resource_id, category, content, rating)
        summary_service.create_resource_summary(resource_id)
        session.commit()

        try:
            resource = comment_service.get_resource(resource_id)
            send_email(
                template_name=config.get(
                    'ckan.feedback.notice.email.template_resource_comment'
                ),
                organization_id=resource.Resource.package.owner_org,
                subject=config.get(
                    'ckan.feedback.notice.email.subject_resource_comment'
                ),
                target_name=resource.Resource.name,
                category=category,
                content=content,
                url=url_for(
                    'resource_comment.comment', resource_id=resource_id, _external=True
                ),
            )
        except Exception:
            log.exception('Send email failed, for feedback notification.')

        helpers.flash_success(
            _(
                'Your comment has been sent.<br>The comment will not be displayed until'
                ' approved by an administrator.'
            ),
            allow_html=True,
        )

        resp = make_response(
            redirect(url_for('resource.read', id=package_name, resource_id=resource_id))
        )

        resp.set_cookie(resource_id, 'alreadyPosted')

        return resp

    # resource_comment/<resource_id>/comment/approve
    @staticmethod
    @check_administrator
    def approve_comment(resource_id):
        ResourceController._check_organization_admin_role(resource_id)
        resource_comment_id = request.form.get('resource_comment_id')
        if not resource_comment_id:
            toolkit.abort(400)

        comment_service.approve_resource_comment(resource_comment_id, current_user.id)
        summary_service.refresh_resource_summary(resource_id)
        session.commit()

        return redirect(url_for('resource_comment.comment', resource_id=resource_id))

    # resource_comment/<resource_id>/comment/reply
    @staticmethod
    @check_administrator
    def reply(resource_id):
        ResourceController._check_organization_admin_role(resource_id)
        resource_comment_id = request.form.get('resource_comment_id', '')
        content = request.form.get('reply_content', '')
        if not (resource_comment_id and content):
            toolkit.abort(400)

        comment_service.create_reply(resource_comment_id, content, current_user.id)
        session.commit()

        return redirect(url_for('resource_comment.comment', resource_id=resource_id))

    @staticmethod
    def _check_organization_admin_role(resource_id):
        resource = comment_service.get_resource(resource_id)
        if (
            not has_organization_admin_role(resource.Resource.package.owner_org)
            and not current_user.sysadmin
        ):
            toolkit.abort(
                404,
                _(
                    'The requested URL was not found on the server. If you entered the'
                    ' URL manually please check your spelling and try again.'
                ),
            )



from unittest.mock import patch

import pytest

from ckanext.feedback.controllers.pagination import get_pagination_value


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestPagination:
    @patch('ckanext.feedback.controllers.pagination._pager_url')
    @patch('ckanext.feedback.controllers.pagination.h.url_for')
    @patch('ckanext.feedback.controllers.pagination.h.get_page_number')
    @patch('ckanext.feedback.controllers.pagination.request.args')
    @patch('ckanext.feedback.controllers.pagination.config')
    def test_get_pagination_value(
        self,
        mock_config,
        mock_request_args,
        mock_get_page_number,
        mock_url_for,
        mock_pager_url,
    ):
        mock_get_page_number.return_value = 1
        mock_request_args.return_value = {'page': 1}
        mock_request_args.items.return_value = [
            ('key1', 'value1'),
            ('key2', 'value2'),
            ('page', 1),
        ]
        mock_config.get.return_value = 20
        mock_url_for.return_value = 'utilization/search'
        mock_pager_url.return_value = (
            'utilization/search?key1=value1&key2=value2&page=2'
        )
        endpoint = 'utilization.search'

        page, limit, offset, pager_url = get_pagination_value(endpoint)

        assert page == 1
        assert limit == 20
        assert offset == 0
        assert pager_url() == 'utilization/search?key1=value1&key2=value2&page=2'



from unittest.mock import MagicMock, patch

import pytest
import requests
from ckan import model
from ckan.tests import factories
from flask import Flask

from ckanext.feedback.command.feedback import (
    create_download_tables,
    create_resource_tables,
    create_utilization_tables,
)
from ckanext.feedback.controllers.download import DownloadController
from ckanext.feedback.models.download import DownloadSummary
from ckanext.feedback.models.session import session


def get_downloads(resource_id):
    count = (
        session.query(DownloadSummary.download)
        .filter(DownloadSummary.resource_id == resource_id)
        .scalar()
    )
    return count


@pytest.mark.usefixtures('clean_db', 'with_plugins', 'with_request_context')
class TestDownloadController:
    @classmethod
    def setup_class(cls):
        model.repo.init_db()
        engine = model.meta.engine
        create_utilization_tables(engine)
        create_resource_tables(engine)
        create_download_tables(engine)

    def setup_method(self, method):
        self.app = Flask(__name__)

    @patch('ckanext.feedback.controllers.download.feedback_config.download_handler')
    @patch('ckanext.feedback.controllers.download.resource.download')
    def test_extended_download(self, mock_download, mock_download_handler):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        mock_download_handler.return_value = None

        with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'document'}):
            DownloadController.extended_download(
                'package_type', resource['package_id'], resource['id'], None
            )
            assert get_downloads(resource['id']) == 1
            mock_download.assert_called_once_with(
                package_type='package_type',
                id=resource['package_id'],
                resource_id=resource['id'],
                filename=resource['url'],
            )

    @patch('ckanext.feedback.controllers.download.resource.download')
    def test_extended_download_with_preview(self, mock_download):
        resource = factories.Resource()
        with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'image'}):
            DownloadController.extended_download(
                'package_type', resource['package_id'], resource['id'], resource['url']
            )
            assert get_downloads(resource['id']) is None
            assert mock_download

    @patch('ckanext.feedback.controllers.download.feedback_config.download_handler')
    @patch('ckanext.feedback.controllers.download.resource.download')
    def test_extended_download_with_set_download_handler(
        self, handler, mock_download_handler
    ):
        resource = factories.Resource()
        mock_download_handler.return_value = handler
        with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'image'}):
            DownloadController.extended_download(
                'package_type', resource['package_id'], resource['id'], resource['url']
            )
            handler.assert_called_once_with(
                package_type='package_type',
                id=resource['package_id'],
                resource_id=resource['id'],
                filename=resource['url'],
            )

    @patch('ckanext.feedback.controllers.download.feedback_config.download_handler')
    @patch('ckanext.feedback.controllers.download.resource.download')
    @patch('ckanext.feedback.controllers.download.requests.get')
    @patch('ckanext.feedback.controllers.download.Response')
    def test_extended_download_get_redirect_resource(
        self, mock_Response, mock_requests_get, mock_download, mock_download_handler
    ):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        mock_download_handler.return_value = None

        mock_responce = MagicMock()
        mock_responce.status_code = 302
        mock_responce.headers.get.return_value = 'http://mock_url.com/mock.txt'
        mock_download.return_value = mock_responce

        mock_requests_get.return_value = MagicMock()

        mock_external_response = MagicMock()
        mock_external_response.status_code = 200
        mock_external_response.headers.get.return_value = 'filename="mock.txt"'
        mock_Response.return_value = mock_external_response

        with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'document'}):
            DownloadController.extended_download(
                'package_type', resource['package_id'], resource['id'], None
            )
            assert get_downloads(resource['id']) == 1
            mock_download.assert_called_once_with(
                package_type='package_type',
                id=resource['package_id'],
                resource_id=resource['id'],
                filename=resource['url'],
            )

    @patch('ckanext.feedback.controllers.download.feedback_config.download_handler')
    @patch('ckanext.feedback.controllers.download.resource.download')
    @patch('ckanext.feedback.controllers.download.requests.get')
    @patch('ckanext.feedback.controllers.download.Response')
    def test_extended_download_get_redirect_resource_non_filename(
        self, mock_Response, mock_requests_get, mock_download, mock_download_handler
    ):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        mock_download_handler.return_value = None

        mock_responce = MagicMock()
        mock_responce.status_code = 302
        mock_responce.headers.get.return_value = 'http://mock_url.com/mock.txt'
        mock_download.return_value = mock_responce

        mock_requests_get.return_value = MagicMock()

        mock_external_response = MagicMock()
        mock_external_response.status_code = 200
        mock_external_response.headers.get.return_value = ''
        mock_Response.return_value = mock_external_response

        with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'document'}):
            DownloadController.extended_download(
                'package_type', resource['package_id'], resource['id'], None
            )
            assert get_downloads(resource['id']) == 1
            mock_download.assert_called_once_with(
                package_type='package_type',
                id=resource['package_id'],
                resource_id=resource['id'],
                filename=resource['url'],
            )

    @patch('ckanext.feedback.controllers.download.feedback_config.download_handler')
    @patch('ckanext.feedback.controllers.download.resource.download')
    @patch('ckanext.feedback.controllers.download.requests.get')
    @patch('ckanext.feedback.controllers.download.Response')
    def test_extended_download_redirect_resource_ConnectionError(
        self, mock_Response, mock_requests_get, mock_download, mock_download_handler
    ):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        mock_download_handler.return_value = None

        mock_responce = MagicMock()
        mock_responce.status_code = 302
        mock_responce.headers.get.return_value = 'http://mock_url.com/mock.txt'
        mock_download.return_value = mock_responce

        mock_requests_get.side_effect = requests.exceptions.ConnectionError()

        mock_external_response = MagicMock()
        mock_external_response.status_code = 200
        mock_external_response.headers.get.return_value = ''
        mock_Response.return_value = mock_external_response

        with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'document'}):
            DownloadController.extended_download(
                'package_type', resource['package_id'], resource['id'], None
            )
            assert get_downloads(resource['id']) == 1
            mock_download.assert_called_once_with(
                package_type='package_type',
                id=resource['package_id'],
                resource_id=resource['id'],
                filename=resource['url'],
            )

    @patch('ckanext.feedback.controllers.download.feedback_config.download_handler')
    @patch('ckanext.feedback.controllers.download.resource.download')
    @patch('ckanext.feedback.controllers.download.requests.get')
    @patch('ckanext.feedback.controllers.download.Response')
    def test_extended_download_without_redirect_resource(
        self, mock_Response, mock_requests_get, mock_download, mock_download_handler
    ):
        owner_org = factories.Organization()
        dataset = factories.Dataset(owner_org=owner_org['id'])
        resource = factories.Resource(package_id=dataset['id'])
        mock_download_handler.return_value = None

        mock_responce = MagicMock()
        mock_responce.status_code = 302
        mock_responce.headers.get.return_value = 'http://mock_url.com/mock.txt'
        mock_download.return_value = mock_responce

        mock_requests_get.return_value = MagicMock()

        mock_external_response = MagicMock()
        mock_external_response.status_code = 500
        mock_external_response.headers.get.return_value = ''
        mock_Response.return_value = mock_external_response

        with self.app.test_request_context(headers={'Sec-Fetch-Dest': 'document'}):
            DownloadController.extended_download(
                'package_type', resource['package_id'], resource['id'], None
            )
            assert get_downloads(resource['id']) == 1
            mock_download.assert_called_once_with(
                package_type='package_type',
                id=resource['package_id'],
                resource_id=resource['id'],
                filename=resource['url'],
            )
